
import copy


class charonLineParserHBBCStrategyLOCA:
    "HBBCStrategyLOCA parser"

    def __init__(self):
        # Register the parsing keys
        self.parserName = "HBBCStrategyLOCA"
        self.primary = "small signal sweep from {potential1} to {potential2}"
        self.options = []
        self.options.append("with amplitude {ssAmplitude}")
        self.options.append("with phase shift {ssPhaseShift}")
        self.options.append("with frequency {ssFrequency}")
        self.foundOptions = []
        for index,opt in enumerate(self.options):
            self.foundOptions.append(False)
        self.primaryVariables = []
        self.optionVariables = [[]]*len(self.options)
        self.interpreterHelpLine = "small signal sweep from {potential1} to {potential2} [with amplitude {ssAmplitude} [with phase shift {ssPhaseShift} [with frequency {ssFrequency}]]] "
        self.interpreterQuickHelp = "Specify a small signal analysis while performing a DC sweep."
        self.interpreterLongHelp = "Specify a small signal analysis while performing a DC sweep. <> potential1 is the beginning voltage in a LOCA sweep <> potential2 is the ending voltage in a LOCA sweep <> ssAmplitude is the small-signal perturbation amplitude <> ssPhaseShift is the small-signal phase shift <> ssFrequency is the small-signal frequency."

        # Register the xml required lines
        self.xmlRequiredLines = []
        self.xmlRequiredLinePriority = []
        self.xmlRequiredLines.append("Charon->Boundary Conditions->{sidesetID}ANONYMOUS->Data,Varying Voltage,string,Parameter")
        self.xmlRequiredLines.append("Charon->Solution Control,Piro Solver,string,LOCA")
        self.xmlRequiredLines.append("Charon->Solution Control->LOCA->Predictor,Method,string,Constant")
        self.xmlRequiredLines.append("Charon->Solution Control->LOCA->Stepper,Initial Value,double,{potential1}")
        self.xmlRequiredLines.append("Charon->Solution Control->LOCA->Stepper,Continuation Parameter,string,Varying Voltage")
        self.xmlRequiredLines.append("Charon->Solution Control->LOCA->Stepper,Max Steps,int,1000")
        self.xmlRequiredLines.append("Charon->Solution Control->LOCA->Stepper,Max Value,double,{potential2}")
        self.xmlRequiredLines.append("Charon->Solution Control->LOCA->Stepper,Min Value,double,{potential1}")
        self.xmlRequiredLines.append("Charon->Solution Control->LOCA->Stepper,Compute Eigenvalues,bool,0")
        self.xmlRequiredLines.append("Charon->Solution Control->LOCA->Step Size,Initial Step Size,double,1.0")
        self.xmlRequiredLines.append("Charon->Active Parameters,Number of Parameter Vectors,int,1")
        self.xmlRequiredLines.append("Charon->Active Parameters->Parameter Vector 0,Number,int,1")
        self.xmlRequiredLines.append("Charon->Active Parameters->Parameter Vector 0,Parameter 0,string,Varying Voltage")
        self.xmlRequiredLines.append("Charon->Active Parameters->Parameter Vector 0,Initial Value 0,double,{potential1}")
        self.xmlRequiredLines.append("use Modifier 0")
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlNewRequiredLines = []

        # Register the xml optional lines
        self.xmlOptionalLines = [[]]
        self.xmlOptionalLinePriority = [[]]
        self.xmlOptionalLines[0].append(" Charon->Boundary Conditions->{sidesetID}ANONYMOUS->Data->Small Signal Perturbation,Amplitude,double,{ssAmplitude}")
        self.xmlOptionalLines.append([])
        self.xmlOptionalLines[1].append(" Charon->Boundary Conditions->{sidesetID}ANONYMOUS->Data->Small Signal Perturbation,Phase Shift,double,{ssPhaseShift}")
        self.xmlOptionalLines.append([])
        self.xmlOptionalLines[2].append(" Charon->Boundary Conditions->{sidesetID}ANONYMOUS->Data->Small Signal Perturbation,Frequency,double,{ssFrequency}")
        self.xmlOptionalLinePriority[0].append(2)
        self.xmlOptionalLinePriority.append([])
        self.xmlOptionalLinePriority[1].append(2)
        self.xmlOptionalLinePriority.append([])
        self.xmlOptionalLinePriority[2].append(2)

        # Register the xml default lines
        self.xmlDefaultLines = []
        self.xmlDefaultLinePriority = []
        self.xmlDefaultLines.append("Charon->Boundary Conditions->{sidesetID}ANONYMOUS->Data->Small Signal Perturbation,Frequency,double,1e0")
        self.xmlDefaultLines.append("Charon->Boundary Conditions->{sidesetID}ANONYMOUS->Data->Small Signal Perturbation,Phase Shift,double,0")
        self.xmlDefaultLinePriority.append(1)
        self.xmlDefaultLinePriority.append(1)

        self.xmlReturned = []
        self.xmlPriorityCode = []
        self.specialInformation = []



    def isThisMe(self,tokenizer,line):
        affirmPrimary = False
        for index,fO in enumerate(self.foundOptions):
            self.foundOptions[index] = False
        self.primaryVariables = {}
        self.optionVariables = []*len(self.options)
        (affirmPrimary,self.foundOptions,self.primaryVariables,self.optionVariables,suggestedSyntax) = tokenizer.isItMe(line,self.primary,self.options)
        return (affirmPrimary,suggestedSyntax)



    def getName(self):
        # Return parser name
         return self.parserName



    def getHelp(self,verbosity):
        # Return help content
        if verbosity.lower() == "long":
            return (self.interpreterHelpLine,self.interpreterLongHelp)
        else:
            return (self.interpreterHelpLine,self.interpreterQuickHelp)



    def generateXML(self,tokenizer,line):
        # Tokenize the line
        lineTokens = tokenizer.tokenize(line)
        self.xmlNewRequiredLines[:] = []
        for xL in self.xmlRequiredLines:
            self.xmlNewRequiredLines.append(xL)

        for key in self.primaryVariables:
            for nRLIndex,nRL in enumerate(self.xmlNewRequiredLines):
                self.xmlNewRequiredLines[nRLIndex] = nRL.replace(key,self.primaryVariables[key])
            for siIndex, siValue in enumerate(self.specialInformation):
                self.specialInformation[siIndex] = [item.replace(key,self.primaryVariables[key]) for item in siValue]

        for index,xmlLine in enumerate(self.xmlNewRequiredLines):
            self.xmlReturned.append(xmlLine)
            self.xmlPriorityCode.append(self.xmlRequiredLinePriority[index]) #required lines have priority code 2
        for optIndex,fO in enumerate(self.foundOptions):
            if fO == True:
                self.Returned=copy.deepcopy(self.xmlOptionalLines[optIndex])
                for key in self.primaryVariables:
                    for retIndex,xOL in enumerate(self.Returned):
                        self.Returned[retIndex] = xOL.replace(key,self.primaryVariables[key])
                for key in self.optionVariables[optIndex]:
                    for retIndex,xOL in enumerate(self.Returned):
                        self.Returned[retIndex] = xOL.replace(key,self.optionVariables[optIndex][key])
                        for siIndex, siValue in enumerate(self.specialInformation):
                            self.specialInformation[siIndex] = [item.replace(key,self.optionVariables[optIndex][key]) for item in siValue]
                for retIndex,ret in enumerate(self.Returned):
                    self.xmlReturned.append(ret)
                    self.xmlPriorityCode.append(self.xmlOptionalLinePriority[optIndex][retIndex])

        for xmlLine in self.xmlDefaultLines:
            self.xmlReturned.append(xmlLine)
            self.xmlPriorityCode.append(1) #optional lines have priority code 1

        return (self.xmlReturned,self.xmlPriorityCode,self.specialInformation)



