
try:
    import coloramaDISABLED as colors
except ImportError:
    class stubColors:
        "subs for colors when colors doesn't exist on system"
    
        def __init__(self):
            self.Fore = colorClass()
            self.Back = colorClass()
            self.Style = styleClass()
    
    class colorClass():
        "stubbed color class"
    
        def __init__(self):
            self.BLACK = ""
            self.BLUE = ""
            self.WHITE = ""
            self.RED = ""
            self.GREEN = ""
    
    class styleClass():
        "stubbed style class"
    
        def __init__(self):
            self.RESET_ALL = ""
    colors = stubColors()

import sys
from .charonLineParserBlockNames import *
from .charonBlockParserPotential import *
from .Potential.PotentialParserLib import *



class HeterojunctionForPotentialParserLib:
    "This is the  HeterojunctionForPotentialParserLib parser library "


    def __init__(self):
        # set the parser library name 
        self.parserLibName = "HeterojunctionForPotentialParserLib"
        # create the linparser objects 
        self.lineParsers = []
        self.lineParsers.append(charonLineParserBlockNames())
        # create the blockparser objects 
        self.blockParsers = []
        self.blockParsers.append([charonBlockParserPotential(),PotentialParserLib()])
        # create the parserLibrary objects 
        parserLibraries = []
        parserLibraries.append(PotentialParserLib())


    def isThisMyLine(self,tokenizer,line):
        suggestedSyntax = []
        for lP in self.lineParsers:
            (self.isThisMe,suggestedSyntaxItem) = lP.isThisMe(tokenizer,line)
            if suggestedSyntaxItem is not "":
                suggestedSyntax.append(suggestedSyntaxItem)
            if self.isThisMe == True:
                return (True,lP,suggestedSyntax)
        return (False,None,suggestedSyntax)


    def isThisMyBlock(self,tokenizer,line):
        for bP in self.blockParsers:
            self.isThisMe = bP[0].isThisMe(tokenizer,line)
            if self.isThisMe == True:
                return (True,bP[0],bP[1])
        return (False,None,None)


    def generateHelp(self,genHelp,indent):
        self.addIndent = "     "
        cRStyle = ""
        for lP in self.lineParsers:
            (self.helpLine,self.helpContent) = lP.getHelp(genHelp)
            self.helpContentList = self.helpContent.split("<>")
            print (cRStyle+indent+colors.Fore.RED+colors.Back.WHITE+self.helpLine)
            cRStyle = "\n"
            for hCL in self.helpContentList:
                print ("\t"+indent+colors.Fore.BLUE+colors.Back.WHITE+hCL.lstrip())
        for bP in range(len(self.blockParsers)):
            print (indent+colors.Fore.GREEN+colors.Back.WHITE+self.blockParsers[bP][0].getHelpLine().lstrip())
            self.blockParsers[bP][1].generateHelp(genHelp,indent+self.addIndent)
            print (indent+colors.Fore.GREEN+colors.Back.WHITE+self.blockParsers[bP][0].getHelpLine().replace("start","end").lstrip())
            print (indent+colors.Style.RESET_ALL)


    def getName(self):
        return self.parserLibName
