
try:
    import coloramaDISABLED as colors
except ImportError:
    class stubColors:
        "subs for colors when colors doesn't exist on system"
    
        def __init__(self):
            self.Fore = colorClass()
            self.Back = colorClass()
            self.Style = styleClass()
    
    class colorClass():
        "stubbed color class"
    
        def __init__(self):
            self.BLACK = ""
            self.BLUE = ""
            self.WHITE = ""
            self.RED = ""
            self.GREEN = ""
    
    class styleClass():
        "stubbed style class"
    
        def __init__(self):
            self.RESET_ALL = ""
    colors = stubColors()

import sys
from .charonLineParserSpatialRange import *
from .charonLineParserDrivingForce import *
from .charonLineParserMinField import *
from .charonLineParserKaneBeta import *
from .charonLineParserKaneA import *
from .charonLineParserHurkxGamma import *
from .charonLineParserBehaviorModel import *
from .charonLineParserKaneAlpha import *
from .charonLineParserHurkxB import *
from .charonLineParserKaneB import *
from .charonLineParserHurkxA import *
from .charonLineParserKaneGamma import *
from .charonLineParserHurkxBeta import *
from .charonLineParserHurkxAlpha import *



class Band2BandTunnelingParserLib:
    "This is the  Band2BandTunnelingParserLib parser library "


    def __init__(self):
        # set the parser library name 
        self.parserLibName = "Band2BandTunnelingParserLib"
        # create the linparser objects 
        self.lineParsers = []
        self.lineParsers.append(charonLineParserSpatialRange())
        self.lineParsers.append(charonLineParserDrivingForce())
        self.lineParsers.append(charonLineParserMinField())
        self.lineParsers.append(charonLineParserKaneBeta())
        self.lineParsers.append(charonLineParserKaneA())
        self.lineParsers.append(charonLineParserHurkxGamma())
        self.lineParsers.append(charonLineParserBehaviorModel())
        self.lineParsers.append(charonLineParserKaneAlpha())
        self.lineParsers.append(charonLineParserHurkxB())
        self.lineParsers.append(charonLineParserKaneB())
        self.lineParsers.append(charonLineParserHurkxA())
        self.lineParsers.append(charonLineParserKaneGamma())
        self.lineParsers.append(charonLineParserHurkxBeta())
        self.lineParsers.append(charonLineParserHurkxAlpha())
        # create the blockparser objects 
        self.blockParsers = []
        # create the parserLibrary objects 
        parserLibraries = []


    def isThisMyLine(self,tokenizer,line):
        suggestedSyntax = []
        for lP in self.lineParsers:
            (self.isThisMe,suggestedSyntaxItem) = lP.isThisMe(tokenizer,line)
            if suggestedSyntaxItem is not "":
                suggestedSyntax.append(suggestedSyntaxItem)
            if self.isThisMe == True:
                return (True,lP,suggestedSyntax)
        return (False,None,suggestedSyntax)


    def isThisMyBlock(self,tokenizer,line):
        for bP in self.blockParsers:
            self.isThisMe = bP[0].isThisMe(tokenizer,line)
            if self.isThisMe == True:
                return (True,bP[0],bP[1])
        return (False,None,None)


    def generateHelp(self,genHelp,indent):
        self.addIndent = "     "
        cRStyle = ""
        for lP in self.lineParsers:
            (self.helpLine,self.helpContent) = lP.getHelp(genHelp)
            self.helpContentList = self.helpContent.split("<>")
            print (cRStyle+indent+colors.Fore.RED+colors.Back.WHITE+self.helpLine)
            cRStyle = "\n"
            for hCL in self.helpContentList:
                print ("\t"+indent+colors.Fore.BLUE+colors.Back.WHITE+hCL.lstrip())
        for bP in range(len(self.blockParsers)):
            print (indent+colors.Fore.GREEN+colors.Back.WHITE+self.blockParsers[bP][0].getHelpLine().lstrip())
            self.blockParsers[bP][1].generateHelp(genHelp,indent+self.addIndent)
            print (indent+colors.Fore.GREEN+colors.Back.WHITE+self.blockParsers[bP][0].getHelpLine().replace("start","end").lstrip())
            print (indent+colors.Style.RESET_ALL)


    def getName(self):
        return self.parserLibName
