
try:
    import coloramaDISABLED as colors
except ImportError:
    class stubColors:
        "subs for colors when colors doesn't exist on system"
    
        def __init__(self):
            self.Fore = colorClass()
            self.Back = colorClass()
            self.Style = styleClass()
    
    class colorClass():
        "stubbed color class"
    
        def __init__(self):
            self.BLACK = ""
            self.BLUE = ""
            self.WHITE = ""
            self.RED = ""
            self.GREEN = ""
    
    class styleClass():
        "stubbed style class"
    
        def __init__(self):
            self.RESET_ALL = ""
    colors = stubColors()

import sys
from .charonLineParserBetax1 import *
from .charonLineParserEg300x0 import *
from .charonLineParserBetax0 import *
from .charonLineParserChi300x1 import *
from .charonLineParserAlphax1 import *
from .charonLineParserEg300x1 import *
from .charonLineParserAlphax0 import *
from .charonLineParserChi300x0 import *
from .charonBlockParserEg300 import *
from .charonBlockParserAlpha import *
from .charonBlockParserChi300 import *
from .charonBlockParserBeta import *
from .Eg300.Eg300ParserLib import *
from .Alpha.AlphaParserLib import *
from .Chi300.Chi300ParserLib import *
from .Beta.BetaParserLib import *



class MoleFractionParametersParserLib:
    "This is the  MoleFractionParametersParserLib parser library "


    def __init__(self):
        # set the parser library name 
        self.parserLibName = "MoleFractionParametersParserLib"
        # create the linparser objects 
        self.lineParsers = []
        self.lineParsers.append(charonLineParserBetax1())
        self.lineParsers.append(charonLineParserEg300x0())
        self.lineParsers.append(charonLineParserBetax0())
        self.lineParsers.append(charonLineParserChi300x1())
        self.lineParsers.append(charonLineParserAlphax1())
        self.lineParsers.append(charonLineParserEg300x1())
        self.lineParsers.append(charonLineParserAlphax0())
        self.lineParsers.append(charonLineParserChi300x0())
        # create the blockparser objects 
        self.blockParsers = []
        self.blockParsers.append([charonBlockParserEg300(),Eg300ParserLib()])
        self.blockParsers.append([charonBlockParserAlpha(),AlphaParserLib()])
        self.blockParsers.append([charonBlockParserChi300(),Chi300ParserLib()])
        self.blockParsers.append([charonBlockParserBeta(),BetaParserLib()])
        # create the parserLibrary objects 
        parserLibraries = []
        parserLibraries.append(Eg300ParserLib())
        parserLibraries.append(AlphaParserLib())
        parserLibraries.append(Chi300ParserLib())
        parserLibraries.append(BetaParserLib())


    def isThisMyLine(self,tokenizer,line):
        suggestedSyntax = []
        for lP in self.lineParsers:
            (self.isThisMe,suggestedSyntaxItem) = lP.isThisMe(tokenizer,line)
            if suggestedSyntaxItem is not "":
                suggestedSyntax.append(suggestedSyntaxItem)
            if self.isThisMe == True:
                return (True,lP,suggestedSyntax)
        return (False,None,suggestedSyntax)


    def isThisMyBlock(self,tokenizer,line):
        for bP in self.blockParsers:
            self.isThisMe = bP[0].isThisMe(tokenizer,line)
            if self.isThisMe == True:
                return (True,bP[0],bP[1])
        return (False,None,None)


    def generateHelp(self,genHelp,indent):
        self.addIndent = "     "
        cRStyle = ""
        for lP in self.lineParsers:
            (self.helpLine,self.helpContent) = lP.getHelp(genHelp)
            self.helpContentList = self.helpContent.split("<>")
            print (cRStyle+indent+colors.Fore.RED+colors.Back.WHITE+self.helpLine)
            cRStyle = "\n"
            for hCL in self.helpContentList:
                print ("\t"+indent+colors.Fore.BLUE+colors.Back.WHITE+hCL.lstrip())
        for bP in range(len(self.blockParsers)):
            print (indent+colors.Fore.GREEN+colors.Back.WHITE+self.blockParsers[bP][0].getHelpLine().lstrip())
            self.blockParsers[bP][1].generateHelp(genHelp,indent+self.addIndent)
            print (indent+colors.Fore.GREEN+colors.Back.WHITE+self.blockParsers[bP][0].getHelpLine().replace("start","end").lstrip())
            print (indent+colors.Style.RESET_ALL)


    def getName(self):
        return self.parserLibName
