
try:
    import coloramaDISABLED as colors
except ImportError:
    class stubColors:
        "subs for colors when colors doesn't exist on system"
    
        def __init__(self):
            self.Fore = colorClass()
            self.Back = colorClass()
            self.Style = styleClass()
    
    class colorClass():
        "stubbed color class"
    
        def __init__(self):
            self.BLACK = ""
            self.BLUE = ""
            self.WHITE = ""
            self.RED = ""
            self.GREEN = ""
    
    class styleClass():
        "stubbed style class"
    
        def __init__(self):
            self.RESET_ALL = ""
    colors = stubColors()

import sys
from .charonLineParserSpatialRange import *
from .charonLineParserThermalVelocityCalculation import *
from .charonLineParserEnergyLevel import *
from .charonLineParserHoleCrossSection import *
from .charonLineParserEnergyWidth import *
from .charonLineParserEnergyDistribution import *
from .charonLineParserTrapDensity import *
from .charonLineParserHoleFieldDependence import *
from .charonLineParserElectronFieldDependence import *
from .charonLineParserTrapType import *
from .charonLineParserElectronPowerDep import *
from .charonLineParserNumberOfLevels import *
from .charonLineParserElectronCrossSection import *
from .charonLineParserDegeneracyFactor import *
from .charonLineParserHolePowerDep import *



class DynamicTrapsParamsParserLib:
    "This is the  DynamicTrapsParamsParserLib parser library "


    def __init__(self):
        # set the parser library name 
        self.parserLibName = "DynamicTrapsParamsParserLib"
        # create the linparser objects 
        self.lineParsers = []
        self.lineParsers.append(charonLineParserSpatialRange())
        self.lineParsers.append(charonLineParserThermalVelocityCalculation())
        self.lineParsers.append(charonLineParserEnergyLevel())
        self.lineParsers.append(charonLineParserHoleCrossSection())
        self.lineParsers.append(charonLineParserEnergyWidth())
        self.lineParsers.append(charonLineParserEnergyDistribution())
        self.lineParsers.append(charonLineParserTrapDensity())
        self.lineParsers.append(charonLineParserHoleFieldDependence())
        self.lineParsers.append(charonLineParserElectronFieldDependence())
        self.lineParsers.append(charonLineParserTrapType())
        self.lineParsers.append(charonLineParserElectronPowerDep())
        self.lineParsers.append(charonLineParserNumberOfLevels())
        self.lineParsers.append(charonLineParserElectronCrossSection())
        self.lineParsers.append(charonLineParserDegeneracyFactor())
        self.lineParsers.append(charonLineParserHolePowerDep())
        # create the blockparser objects 
        self.blockParsers = []
        # create the parserLibrary objects 
        parserLibraries = []


    def isThisMyLine(self,tokenizer,line):
        suggestedSyntax = []
        for lP in self.lineParsers:
            (self.isThisMe,suggestedSyntaxItem) = lP.isThisMe(tokenizer,line)
            if suggestedSyntaxItem is not "":
                suggestedSyntax.append(suggestedSyntaxItem)
            if self.isThisMe == True:
                return (True,lP,suggestedSyntax)
        return (False,None,suggestedSyntax)


    def isThisMyBlock(self,tokenizer,line):
        for bP in self.blockParsers:
            self.isThisMe = bP[0].isThisMe(tokenizer,line)
            if self.isThisMe == True:
                return (True,bP[0],bP[1])
        return (False,None,None)


    def generateHelp(self,genHelp,indent):
        self.addIndent = "     "
        cRStyle = ""
        for lP in self.lineParsers:
            (self.helpLine,self.helpContent) = lP.getHelp(genHelp)
            self.helpContentList = self.helpContent.split("<>")
            print (cRStyle+indent+colors.Fore.RED+colors.Back.WHITE+self.helpLine)
            cRStyle = "\n"
            for hCL in self.helpContentList:
                print ("\t"+indent+colors.Fore.BLUE+colors.Back.WHITE+hCL.lstrip())
        for bP in range(len(self.blockParsers)):
            print (indent+colors.Fore.GREEN+colors.Back.WHITE+self.blockParsers[bP][0].getHelpLine().lstrip())
            self.blockParsers[bP][1].generateHelp(genHelp,indent+self.addIndent)
            print (indent+colors.Fore.GREEN+colors.Back.WHITE+self.blockParsers[bP][0].getHelpLine().replace("start","end").lstrip())
            print (indent+colors.Style.RESET_ALL)


    def getName(self):
        return self.parserLibName
