
try:
    import coloramaDISABLED as colors
except ImportError:
    class stubColors:
        "subs for colors when colors doesn't exist on system"
    
        def __init__(self):
            self.Fore = colorClass()
            self.Back = colorClass()
            self.Style = styleClass()
    
    class colorClass():
        "stubbed color class"
    
        def __init__(self):
            self.BLACK = ""
            self.BLUE = ""
            self.WHITE = ""
            self.RED = ""
            self.GREEN = ""
    
    class styleClass():
        "stubbed style class"
    
        def __init__(self):
            self.RESET_ALL = ""
    colors = stubColors()

import sys
from .charonLineParserNv300x0 import *
from .charonLineParserNc300x0 import *
from .charonLineParserNv_Fx0 import *
from .charonLineParserNv300x1 import *
from .charonLineParserNc_Fx0 import *
from .charonLineParserNv_Fx1 import *
from .charonLineParserNc_Fx1 import *
from .charonLineParserNc300x1 import *
from .charonBlockParserNc_F import *
from .charonBlockParserNv300 import *
from .charonBlockParserNv_F import *
from .charonBlockParserNc300 import *
from .Nc_F.Nc_FParserLib import *
from .Nv300.Nv300ParserLib import *
from .Nv_F.Nv_FParserLib import *
from .Nc300.Nc300ParserLib import *



class MoleFractionParametersParserLib:
    "This is the  MoleFractionParametersParserLib parser library "


    def __init__(self):
        # set the parser library name 
        self.parserLibName = "MoleFractionParametersParserLib"
        # create the linparser objects 
        self.lineParsers = []
        self.lineParsers.append(charonLineParserNv300x0())
        self.lineParsers.append(charonLineParserNc300x0())
        self.lineParsers.append(charonLineParserNv_Fx0())
        self.lineParsers.append(charonLineParserNv300x1())
        self.lineParsers.append(charonLineParserNc_Fx0())
        self.lineParsers.append(charonLineParserNv_Fx1())
        self.lineParsers.append(charonLineParserNc_Fx1())
        self.lineParsers.append(charonLineParserNc300x1())
        # create the blockparser objects 
        self.blockParsers = []
        self.blockParsers.append([charonBlockParserNc_F(),Nc_FParserLib()])
        self.blockParsers.append([charonBlockParserNv300(),Nv300ParserLib()])
        self.blockParsers.append([charonBlockParserNv_F(),Nv_FParserLib()])
        self.blockParsers.append([charonBlockParserNc300(),Nc300ParserLib()])
        # create the parserLibrary objects 
        parserLibraries = []
        parserLibraries.append(Nc_FParserLib())
        parserLibraries.append(Nv300ParserLib())
        parserLibraries.append(Nv_FParserLib())
        parserLibraries.append(Nc300ParserLib())


    def isThisMyLine(self,tokenizer,line):
        suggestedSyntax = []
        for lP in self.lineParsers:
            (self.isThisMe,suggestedSyntaxItem) = lP.isThisMe(tokenizer,line)
            if suggestedSyntaxItem is not "":
                suggestedSyntax.append(suggestedSyntaxItem)
            if self.isThisMe == True:
                return (True,lP,suggestedSyntax)
        return (False,None,suggestedSyntax)


    def isThisMyBlock(self,tokenizer,line):
        for bP in self.blockParsers:
            self.isThisMe = bP[0].isThisMe(tokenizer,line)
            if self.isThisMe == True:
                return (True,bP[0],bP[1])
        return (False,None,None)


    def generateHelp(self,genHelp,indent):
        self.addIndent = "     "
        cRStyle = ""
        for lP in self.lineParsers:
            (self.helpLine,self.helpContent) = lP.getHelp(genHelp)
            self.helpContentList = self.helpContent.split("<>")
            print (cRStyle+indent+colors.Fore.RED+colors.Back.WHITE+self.helpLine)
            cRStyle = "\n"
            for hCL in self.helpContentList:
                print ("\t"+indent+colors.Fore.BLUE+colors.Back.WHITE+hCL.lstrip())
        for bP in range(len(self.blockParsers)):
            print (indent+colors.Fore.GREEN+colors.Back.WHITE+self.blockParsers[bP][0].getHelpLine().lstrip())
            self.blockParsers[bP][1].generateHelp(genHelp,indent+self.addIndent)
            print (indent+colors.Fore.GREEN+colors.Back.WHITE+self.blockParsers[bP][0].getHelpLine().replace("start","end").lstrip())
            print (indent+colors.Style.RESET_ALL)


    def getName(self):
        return self.parserLibName
