
import copy


class charonLineParserDopingConcentrationRange:
    "DopingConcentrationRange parser"

    def __init__(self):
        # Register the parsing keys
        self.parserName = "DopingConcentrationRange"
        self.primary = "concentration range is {minimumConcentration} to {maximumConcentration}"
        self.options = []
        self.foundOptions = []
        for index,opt in enumerate(self.options):
            self.foundOptions.append(False)
        self.primaryVariables = []
        self.optionVariables = [[]]*len(self.options)
        self.interpreterHelpLine = "Concentration range is {minimumConcentration} to {maximumConcentration} "
        self.interpreterQuickHelp = "set the range of the magnitude of the dopant concentration"
        self.interpreterLongHelp = "set the range of the magnitude of the dopant concentration <> minimumConcentration is the minimum numerical value in 1/cm^3 <> maximumConcentration is the maximum numerical value in 1/cm^3"

        # Register the xml required lines
        self.xmlRequiredLines = []
        self.xmlRequiredLinePriority = []
        self.xmlRequiredLines.append("Charon->Closure Models->{MaterialBlockName}->Doping->{functionName},Doping Min Value,double,{minimumConcentration}")
        self.xmlRequiredLines.append("Charon->Closure Models->{MaterialBlockName}->Doping->{functionName},Doping Max Value,double,{maximumConcentration}")
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlNewRequiredLines = []

        # Register the xml optional lines
        self.xmlOptionalLines = [[]]
        self.xmlOptionalLinePriority = [[]]

        # Register the xml default lines
        self.xmlDefaultLines = []
        self.xmlDefaultLinePriority = []

        self.xmlReturned = []
        self.xmlPriorityCode = []
        self.specialInformation = []



    def isThisMe(self,tokenizer,line):
        affirmPrimary = False
        for index,fO in enumerate(self.foundOptions):
            self.foundOptions[index] = False
        self.primaryVariables = {}
        self.optionVariables = []*len(self.options)
        (affirmPrimary,self.foundOptions,self.primaryVariables,self.optionVariables,suggestedSyntax) = tokenizer.isItMe(line,self.primary,self.options)
        return (affirmPrimary,suggestedSyntax)



    def getName(self):
        # Return parser name
         return self.parserName



    def getHelp(self,verbosity):
        # Return help content
        if verbosity.lower() == "long":
            return (self.interpreterHelpLine,self.interpreterLongHelp)
        else:
            return (self.interpreterHelpLine,self.interpreterQuickHelp)



    def generateXML(self,tokenizer,line):
        # Tokenize the line
        lineTokens = tokenizer.tokenize(line)
        self.xmlNewRequiredLines[:] = []
        for xL in self.xmlRequiredLines:
            self.xmlNewRequiredLines.append(xL)

        for key in self.primaryVariables:
            for nRLIndex,nRL in enumerate(self.xmlNewRequiredLines):
                self.xmlNewRequiredLines[nRLIndex] = nRL.replace(key,self.primaryVariables[key])
            for siIndex, siValue in enumerate(self.specialInformation):
                self.specialInformation[siIndex] = [item.replace(key,self.primaryVariables[key]) for item in siValue]

        for index,xmlLine in enumerate(self.xmlNewRequiredLines):
            self.xmlReturned.append(xmlLine)
            self.xmlPriorityCode.append(self.xmlRequiredLinePriority[index]) #required lines have priority code 2
        for optIndex,fO in enumerate(self.foundOptions):
            if fO == True:
                self.Returned=copy.deepcopy(self.xmlOptionalLines[optIndex])
                for key in self.primaryVariables:
                    for retIndex,xOL in enumerate(self.Returned):
                        self.Returned[retIndex] = xOL.replace(key,self.primaryVariables[key])
                for key in self.optionVariables[optIndex]:
                    for retIndex,xOL in enumerate(self.Returned):
                        self.Returned[retIndex] = xOL.replace(key,self.optionVariables[optIndex][key])
                        for siIndex, siValue in enumerate(self.specialInformation):
                            self.specialInformation[siIndex] = [item.replace(key,self.optionVariables[optIndex][key]) for item in siValue]
                for retIndex,ret in enumerate(self.Returned):
                    self.xmlReturned.append(ret)
                    self.xmlPriorityCode.append(self.xmlOptionalLinePriority[optIndex][retIndex])

        for xmlLine in self.xmlDefaultLines:
            self.xmlReturned.append(xmlLine)
            self.xmlPriorityCode.append(1) #optional lines have priority code 1

        return (self.xmlReturned,self.xmlPriorityCode,self.specialInformation)



