
import copy


class charonLineParserMoleFractionRange:
    "MoleFractionRange parser"

    def __init__(self):
        # Register the parsing keys
        self.parserName = "MoleFractionRange"
        self.primary = "spatial range is {locMin} to {locMax}"
        self.options = []
        self.options.append("in x")
        self.options.append("in y")
        self.options.append("in z")
        self.foundOptions = []
        for index,opt in enumerate(self.options):
            self.foundOptions.append(False)
        self.primaryVariables = []
        self.optionVariables = [[]]*len(self.options)
        self.interpreterHelpLine = "spatial range is {locMin} to {locMax} [ in x [ in y [ in z]]] "
        self.interpreterQuickHelp = "Set the coordinate range of the gaussian mole fraction in x  y or z"
        self.interpreterLongHelp = "Set the coordinate range of the gaussian mole fraction in x  y or z"

        # Register the xml required lines
        self.xmlRequiredLines = []
        self.xmlRequiredLinePriority = []
        self.xmlNewRequiredLines = []

        # Register the xml optional lines
        self.xmlOptionalLines = [[]]
        self.xmlOptionalLinePriority = [[]]
        self.xmlOptionalLines[0].append(" Charon->Closure Models->{MaterialBlockName}->Mole Fraction->{functionName},Xmin,double,{locMin}")
        self.xmlOptionalLines[0].append(" Charon->Closure Models->{MaterialBlockName}->Mole Fraction->{functionName},Xmax,double,{locMax}")
        self.xmlOptionalLines.append([])
        self.xmlOptionalLines[1].append(" Charon->Closure Models->{MaterialBlockName}->Mole Fraction->{functionName},Ymin,double,{locMin}")
        self.xmlOptionalLines[1].append(" Charon->Closure Models->{MaterialBlockName}->Mole Fraction->{functionName},Ymax,double,{locMax}")
        self.xmlOptionalLines.append([])
        self.xmlOptionalLines[2].append(" Charon->Closure Models->{MaterialBlockName}->Mole Fraction->{functionName},Zmin,double,{locMin}")
        self.xmlOptionalLines[2].append(" Charon->Closure Models->{MaterialBlockName}->Mole Fraction->{functionName},Zmax,double,{locMax}")
        self.xmlOptionalLinePriority[0].append(2)
        self.xmlOptionalLinePriority[0].append(2)
        self.xmlOptionalLinePriority.append([])
        self.xmlOptionalLinePriority[1].append(2)
        self.xmlOptionalLinePriority[1].append(2)
        self.xmlOptionalLinePriority.append([])
        self.xmlOptionalLinePriority[2].append(2)
        self.xmlOptionalLinePriority[2].append(2)

        # Register the xml default lines
        self.xmlDefaultLines = []
        self.xmlDefaultLinePriority = []

        self.xmlReturned = []
        self.xmlPriorityCode = []
        self.specialInformation = []



    def isThisMe(self,tokenizer,line):
        affirmPrimary = False
        for index,fO in enumerate(self.foundOptions):
            self.foundOptions[index] = False
        self.primaryVariables = {}
        self.optionVariables = []*len(self.options)
        (affirmPrimary,self.foundOptions,self.primaryVariables,self.optionVariables,suggestedSyntax) = tokenizer.isItMe(line,self.primary,self.options)
        return (affirmPrimary,suggestedSyntax)



    def getName(self):
        # Return parser name
         return self.parserName



    def getHelp(self,verbosity):
        # Return help content
        if verbosity.lower() == "long":
            return (self.interpreterHelpLine,self.interpreterLongHelp)
        else:
            return (self.interpreterHelpLine,self.interpreterQuickHelp)



    def generateXML(self,tokenizer,line):
        # Tokenize the line
        lineTokens = tokenizer.tokenize(line)
        self.xmlNewRequiredLines[:] = []
        for xL in self.xmlRequiredLines:
            self.xmlNewRequiredLines.append(xL)

        for key in self.primaryVariables:
            for nRLIndex,nRL in enumerate(self.xmlNewRequiredLines):
                self.xmlNewRequiredLines[nRLIndex] = nRL.replace(key,self.primaryVariables[key])
            for siIndex, siValue in enumerate(self.specialInformation):
                self.specialInformation[siIndex] = [item.replace(key,self.primaryVariables[key]) for item in siValue]

        for index,xmlLine in enumerate(self.xmlNewRequiredLines):
            self.xmlReturned.append(xmlLine)
            self.xmlPriorityCode.append(self.xmlRequiredLinePriority[index]) #required lines have priority code 2
        for optIndex,fO in enumerate(self.foundOptions):
            if fO == True:
                self.Returned=copy.deepcopy(self.xmlOptionalLines[optIndex])
                for key in self.primaryVariables:
                    for retIndex,xOL in enumerate(self.Returned):
                        self.Returned[retIndex] = xOL.replace(key,self.primaryVariables[key])
                for key in self.optionVariables[optIndex]:
                    for retIndex,xOL in enumerate(self.Returned):
                        self.Returned[retIndex] = xOL.replace(key,self.optionVariables[optIndex][key])
                        for siIndex, siValue in enumerate(self.specialInformation):
                            self.specialInformation[siIndex] = [item.replace(key,self.optionVariables[optIndex][key]) for item in siValue]
                for retIndex,ret in enumerate(self.Returned):
                    self.xmlReturned.append(ret)
                    self.xmlPriorityCode.append(self.xmlOptionalLinePriority[optIndex][retIndex])

        for xmlLine in self.xmlDefaultLines:
            self.xmlReturned.append(xmlLine)
            self.xmlPriorityCode.append(1) #optional lines have priority code 1

        return (self.xmlReturned,self.xmlPriorityCode,self.specialInformation)



