
import copy


class charonLineParserCaptureCrossSection:
    "CaptureCrossSection parser"

    def __init__(self):
        # Register the parsing keys
        self.parserName = "CaptureCrossSection"
        self.primary = "capture cross section"
        self.options = []
        self.options.append("= {sigma}")
        self.options.append("swept from {sigma1} to {sigma2}")
        self.foundOptions = []
        for index,opt in enumerate(self.options):
            self.foundOptions.append(False)
        self.primaryVariables = []
        self.optionVariables = [[]]*len(self.options)
        self.interpreterHelpLine = "capture cross section [ = {sigma} [ swept from {sigma1} to {sigma2} ]] "
        self.interpreterQuickHelp = "Sets the interface trap capture cross section for the TID model"
        self.interpreterLongHelp = "Sets the interface trap capture cross section for the TID model"

        # Register the xml required lines
        self.xmlRequiredLines = []
        self.xmlRequiredLinePriority = []
        self.xmlNewRequiredLines = []

        # Register the xml optional lines
        self.xmlOptionalLines = [[]]
        self.xmlOptionalLinePriority = [[]]
        self.xmlOptionalLines[0].append(" Charon->Closure Models->{MaterialBlockName}->TID->Kimpton Model->Interface Traps,Capture Cross Section,double,{sigma}")
        self.xmlOptionalLines.append([])
        self.xmlOptionalLines[1].append(" Charon->Closure Models->{MaterialBlockName}->TID->Kimpton Model->Interface Traps,Capture Cross Section Sweep is On,bool,false")
        self.xmlOptionalLines[1].append(" Charon->Closure Models->{MaterialBlockName}->TID->Kimpton Model->Interface Traps,Capture Cross Section Sweep is On,bool,true")
        self.xmlOptionalLines[1].append(" Charon->Closure Models->{MaterialBlockName}->TID->Kimpton Model->Interface Traps,Capture Cross Section Sweep,string,Parameter")
        self.xmlOptionalLines[1].append(" Charon->Closure Models->{MaterialBlockName}->TID->Kimpton Model->Interface Traps,Initial Capture Cross Section,double,{sigma1}")
        self.xmlOptionalLines[1].append(" Charon->Closure Models->{MaterialBlockName}->TID->Kimpton Model->Interface Traps,Final Capture Cross Section,double,{sigma2}")
        self.xmlOptionalLines[1].append(" Charon->Solution Control,Piro Solver,string,LOCA")
        self.xmlOptionalLines[1].append(" Charon->Solution Control->LOCA->Predictor,Method,string,Constant")
        self.xmlOptionalLines[1].append(" Charon->Solution Control->LOCA->Stepper,Initial Value,double,0.0")
        self.xmlOptionalLines[1].append(" Charon->Solution Control->LOCA->Stepper,Continuation Parameter,string,Interface Trap Capture Cross Section Sweep")
        self.xmlOptionalLines[1].append(" Charon->Solution Control->LOCA->Stepper,Max Steps,int,1000")
        self.xmlOptionalLines[1].append(" Charon->Solution Control->LOCA->Stepper,Max Value,double,1.0")
        self.xmlOptionalLines[1].append(" Charon->Solution Control->LOCA->Stepper,Min Value,double,0.0")
        self.xmlOptionalLines[1].append(" Charon->Solution Control->LOCA->Stepper,Compute Eigenvalues,bool,0")
        self.xmlOptionalLines[1].append(" Charon->Solution Control->LOCA->Step Size,Initial Step Size,double,0.05")
        self.xmlOptionalLines[1].append(" Charon->Active Parameters,Number of Parameter Vectors,int,1")
        self.xmlOptionalLines[1].append(" Charon->Active Parameters->Parameter Vector 0,Number,int,1")
        self.xmlOptionalLines[1].append(" Charon->Active Parameters->Parameter Vector 0,Parameter 0,string,Interface Trap Capture Cross Section Sweep")
        self.xmlOptionalLines[1].append(" Charon->Active Parameters->Parameter Vector 0,Initial Value 0,double,0.0")
        self.xmlOptionalLinePriority[0].append(2)
        self.xmlOptionalLinePriority.append([])
        self.xmlOptionalLinePriority[1].append(2)
        self.xmlOptionalLinePriority[1].append(2)
        self.xmlOptionalLinePriority[1].append(2)
        self.xmlOptionalLinePriority[1].append(2)
        self.xmlOptionalLinePriority[1].append(2)
        self.xmlOptionalLinePriority[1].append(2)
        self.xmlOptionalLinePriority[1].append(2)
        self.xmlOptionalLinePriority[1].append(2)
        self.xmlOptionalLinePriority[1].append(2)
        self.xmlOptionalLinePriority[1].append(2)
        self.xmlOptionalLinePriority[1].append(2)
        self.xmlOptionalLinePriority[1].append(2)
        self.xmlOptionalLinePriority[1].append(2)
        self.xmlOptionalLinePriority[1].append(2)
        self.xmlOptionalLinePriority[1].append(2)
        self.xmlOptionalLinePriority[1].append(2)
        self.xmlOptionalLinePriority[1].append(2)
        self.xmlOptionalLinePriority[1].append(2)

        # Register the xml default lines
        self.xmlDefaultLines = []
        self.xmlDefaultLinePriority = []

        self.xmlReturned = []
        self.xmlPriorityCode = []
        self.specialInformation = []



    def isThisMe(self,tokenizer,line):
        affirmPrimary = False
        for index,fO in enumerate(self.foundOptions):
            self.foundOptions[index] = False
        self.primaryVariables = {}
        self.optionVariables = []*len(self.options)
        (affirmPrimary,self.foundOptions,self.primaryVariables,self.optionVariables,suggestedSyntax) = tokenizer.isItMe(line,self.primary,self.options)
        return (affirmPrimary,suggestedSyntax)



    def getName(self):
        # Return parser name
         return self.parserName



    def getHelp(self,verbosity):
        # Return help content
        if verbosity.lower() == "long":
            return (self.interpreterHelpLine,self.interpreterLongHelp)
        else:
            return (self.interpreterHelpLine,self.interpreterQuickHelp)



    def generateXML(self,tokenizer,line):
        # Tokenize the line
        lineTokens = tokenizer.tokenize(line)
        self.xmlNewRequiredLines[:] = []
        for xL in self.xmlRequiredLines:
            self.xmlNewRequiredLines.append(xL)

        for key in self.primaryVariables:
            for nRLIndex,nRL in enumerate(self.xmlNewRequiredLines):
                self.xmlNewRequiredLines[nRLIndex] = nRL.replace(key,self.primaryVariables[key])
            for siIndex, siValue in enumerate(self.specialInformation):
                self.specialInformation[siIndex] = [item.replace(key,self.primaryVariables[key]) for item in siValue]

        for index,xmlLine in enumerate(self.xmlNewRequiredLines):
            self.xmlReturned.append(xmlLine)
            self.xmlPriorityCode.append(self.xmlRequiredLinePriority[index]) #required lines have priority code 2
        for optIndex,fO in enumerate(self.foundOptions):
            if fO == True:
                self.Returned=copy.deepcopy(self.xmlOptionalLines[optIndex])
                for key in self.primaryVariables:
                    for retIndex,xOL in enumerate(self.Returned):
                        self.Returned[retIndex] = xOL.replace(key,self.primaryVariables[key])
                for key in self.optionVariables[optIndex]:
                    for retIndex,xOL in enumerate(self.Returned):
                        self.Returned[retIndex] = xOL.replace(key,self.optionVariables[optIndex][key])
                        for siIndex, siValue in enumerate(self.specialInformation):
                            self.specialInformation[siIndex] = [item.replace(key,self.optionVariables[optIndex][key]) for item in siValue]
                for retIndex,ret in enumerate(self.Returned):
                    self.xmlReturned.append(ret)
                    self.xmlPriorityCode.append(self.xmlOptionalLinePriority[optIndex][retIndex])

        for xmlLine in self.xmlDefaultLines:
            self.xmlReturned.append(xmlLine)
            self.xmlPriorityCode.append(1) #optional lines have priority code 1

        return (self.xmlReturned,self.xmlPriorityCode,self.specialInformation)



