
import copy


class charonLineParserGaussDecay:
    "GaussDecay parser"

    def __init__(self):
        # Register the parsing keys
        self.parserName = "GaussDecay"
        self.primary = "gauss decay with start position at {positionVal} and decay width at {widthVal}"
        self.options = []
        self.options.append("in +x")
        self.options.append("in -x")
        self.options.append("in +y")
        self.options.append("in -y")
        self.options.append("in +z")
        self.options.append("in -z")
        self.foundOptions = []
        for index,opt in enumerate(self.options):
            self.foundOptions.append(False)
        self.primaryVariables = []
        self.optionVariables = [[]]*len(self.options)
        self.interpreterHelpLine = "gauss decay with start position at {positionVal} and decay width at {widthVal} [ in +x [ in -x [in +y [in -y [in +z [in -z]]]]]] "
        self.interpreterQuickHelp = "Specify Gauss decay for a Uniform doping profile."
        self.interpreterLongHelp = "Specify Gauss decay for a Uniform doping and a total of six decays along each +/- axis are allowed."

        # Register the xml required lines
        self.xmlRequiredLines = []
        self.xmlRequiredLinePriority = []
        self.xmlNewRequiredLines = []

        # Register the xml optional lines
        self.xmlOptionalLines = [[]]
        self.xmlOptionalLinePriority = [[]]
        self.xmlOptionalLines[0].append(" Charon->Closure Models->{MaterialBlockName}->Doping->{functionName}->Gauss Decay,Decay Direction 1,string,X Positive")
        self.xmlOptionalLines[0].append(" Charon->Closure Models->{MaterialBlockName}->Doping->{functionName}->Gauss Decay,Decay Position 1,double,{positionVal}")
        self.xmlOptionalLines[0].append(" Charon->Closure Models->{MaterialBlockName}->Doping->{functionName}->Gauss Decay,Decay Width 1,double,{widthVal}")
        self.xmlOptionalLines.append([])
        self.xmlOptionalLines[1].append(" Charon->Closure Models->{MaterialBlockName}->Doping->{functionName}->Gauss Decay,Decay Direction 2,string,X Negative")
        self.xmlOptionalLines[1].append(" Charon->Closure Models->{MaterialBlockName}->Doping->{functionName}->Gauss Decay,Decay Position 2,double,{positionVal}")
        self.xmlOptionalLines[1].append(" Charon->Closure Models->{MaterialBlockName}->Doping->{functionName}->Gauss Decay,Decay Width 2,double,{widthVal}")
        self.xmlOptionalLines.append([])
        self.xmlOptionalLines[2].append(" Charon->Closure Models->{MaterialBlockName}->Doping->{functionName}->Gauss Decay,Decay Direction 3,string,Y Positive")
        self.xmlOptionalLines[2].append(" Charon->Closure Models->{MaterialBlockName}->Doping->{functionName}->Gauss Decay,Decay Position 3,double,{positionVal}")
        self.xmlOptionalLines[2].append(" Charon->Closure Models->{MaterialBlockName}->Doping->{functionName}->Gauss Decay,Decay Width 3,double,{widthVal}")
        self.xmlOptionalLines.append([])
        self.xmlOptionalLines[3].append(" Charon->Closure Models->{MaterialBlockName}->Doping->{functionName}->Gauss Decay,Decay Direction 4,string,Y Negative")
        self.xmlOptionalLines[3].append(" Charon->Closure Models->{MaterialBlockName}->Doping->{functionName}->Gauss Decay,Decay Position 4,double,{positionVal}")
        self.xmlOptionalLines[3].append(" Charon->Closure Models->{MaterialBlockName}->Doping->{functionName}->Gauss Decay,Decay Width 4,double,{widthVal}")
        self.xmlOptionalLines.append([])
        self.xmlOptionalLines[4].append(" Charon->Closure Models->{MaterialBlockName}->Doping->{functionName}->Gauss Decay,Decay Direction 5,string,Z Positive")
        self.xmlOptionalLines[4].append(" Charon->Closure Models->{MaterialBlockName}->Doping->{functionName}->Gauss Decay,Decay Position 5,double,{positionVal}")
        self.xmlOptionalLines[4].append(" Charon->Closure Models->{MaterialBlockName}->Doping->{functionName}->Gauss Decay,Decay Width 5,double,{widthVal}")
        self.xmlOptionalLines.append([])
        self.xmlOptionalLines[5].append(" Charon->Closure Models->{MaterialBlockName}->Doping->{functionName}->Gauss Decay,Decay Direction 6,string,Z Negative")
        self.xmlOptionalLines[5].append(" Charon->Closure Models->{MaterialBlockName}->Doping->{functionName}->Gauss Decay,Decay Position 6,double,{positionVal}")
        self.xmlOptionalLines[5].append(" Charon->Closure Models->{MaterialBlockName}->Doping->{functionName}->Gauss Decay,Decay Width 6,double,{widthVal}")
        self.xmlOptionalLinePriority[0].append(2)
        self.xmlOptionalLinePriority[0].append(2)
        self.xmlOptionalLinePriority[0].append(2)
        self.xmlOptionalLinePriority.append([])
        self.xmlOptionalLinePriority[1].append(2)
        self.xmlOptionalLinePriority[1].append(2)
        self.xmlOptionalLinePriority[1].append(2)
        self.xmlOptionalLinePriority.append([])
        self.xmlOptionalLinePriority[2].append(2)
        self.xmlOptionalLinePriority[2].append(2)
        self.xmlOptionalLinePriority[2].append(2)
        self.xmlOptionalLinePriority.append([])
        self.xmlOptionalLinePriority[3].append(2)
        self.xmlOptionalLinePriority[3].append(2)
        self.xmlOptionalLinePriority[3].append(2)
        self.xmlOptionalLinePriority.append([])
        self.xmlOptionalLinePriority[4].append(2)
        self.xmlOptionalLinePriority[4].append(2)
        self.xmlOptionalLinePriority[4].append(2)
        self.xmlOptionalLinePriority.append([])
        self.xmlOptionalLinePriority[5].append(2)
        self.xmlOptionalLinePriority[5].append(2)
        self.xmlOptionalLinePriority[5].append(2)

        # Register the xml default lines
        self.xmlDefaultLines = []
        self.xmlDefaultLinePriority = []

        self.xmlReturned = []
        self.xmlPriorityCode = []
        self.specialInformation = []



    def isThisMe(self,tokenizer,line):
        affirmPrimary = False
        for index,fO in enumerate(self.foundOptions):
            self.foundOptions[index] = False
        self.primaryVariables = {}
        self.optionVariables = []*len(self.options)
        (affirmPrimary,self.foundOptions,self.primaryVariables,self.optionVariables,suggestedSyntax) = tokenizer.isItMe(line,self.primary,self.options)
        return (affirmPrimary,suggestedSyntax)



    def getName(self):
        # Return parser name
         return self.parserName



    def getHelp(self,verbosity):
        # Return help content
        if verbosity.lower() == "long":
            return (self.interpreterHelpLine,self.interpreterLongHelp)
        else:
            return (self.interpreterHelpLine,self.interpreterQuickHelp)



    def generateXML(self,tokenizer,line):
        # Tokenize the line
        lineTokens = tokenizer.tokenize(line)
        self.xmlNewRequiredLines[:] = []
        for xL in self.xmlRequiredLines:
            self.xmlNewRequiredLines.append(xL)

        for key in self.primaryVariables:
            for nRLIndex,nRL in enumerate(self.xmlNewRequiredLines):
                self.xmlNewRequiredLines[nRLIndex] = nRL.replace(key,self.primaryVariables[key])
            for siIndex, siValue in enumerate(self.specialInformation):
                self.specialInformation[siIndex] = [item.replace(key,self.primaryVariables[key]) for item in siValue]

        for index,xmlLine in enumerate(self.xmlNewRequiredLines):
            self.xmlReturned.append(xmlLine)
            self.xmlPriorityCode.append(self.xmlRequiredLinePriority[index]) #required lines have priority code 2
        for optIndex,fO in enumerate(self.foundOptions):
            if fO == True:
                self.Returned=copy.deepcopy(self.xmlOptionalLines[optIndex])
                for key in self.primaryVariables:
                    for retIndex,xOL in enumerate(self.Returned):
                        self.Returned[retIndex] = xOL.replace(key,self.primaryVariables[key])
                for key in self.optionVariables[optIndex]:
                    for retIndex,xOL in enumerate(self.Returned):
                        self.Returned[retIndex] = xOL.replace(key,self.optionVariables[optIndex][key])
                        for siIndex, siValue in enumerate(self.specialInformation):
                            self.specialInformation[siIndex] = [item.replace(key,self.optionVariables[optIndex][key]) for item in siValue]
                for retIndex,ret in enumerate(self.Returned):
                    self.xmlReturned.append(ret)
                    self.xmlPriorityCode.append(self.xmlOptionalLinePriority[optIndex][retIndex])

        for xmlLine in self.xmlDefaultLines:
            self.xmlReturned.append(xmlLine)
            self.xmlPriorityCode.append(1) #optional lines have priority code 1

        return (self.xmlReturned,self.xmlPriorityCode,self.specialInformation)



