
import copy


class charonLineParserDiscretizationType:
    "DiscretizationType parser"

    def __init__(self):
        # Register the parsing keys
        self.parserName = "DiscretizationType"
        self.primary = "standard discretization type is"
        self.options = []
        self.options.append("drift diffusion gfem")
        self.options.append("drift diffusion effpg")
        self.options.append("nlp")
        self.options.append("nlp gfem")
        self.options.append("nlp cvfem")
        self.options.append("drift diffusion cvfem")
        self.options.append("laplace gfem")
        self.options.append("laplace cvfem")
        self.options.append("lattice gfem")
        self.options.append("ddion gfem")
        self.options.append("ddlattice")
        self.options.append("ddionlattice")
        self.foundOptions = []
        for index,opt in enumerate(self.options):
            self.foundOptions.append(False)
        self.primaryVariables = []
        self.optionVariables = [[]]*len(self.options)
        self.interpreterHelpLine = "standard discretization type is [ drift diffusion gfem [ drift diffusion effpg [ nlp [ nlp gfem [ nlp cvfem [ drift diffusion cvfem [ laplace gfem [ laplace cvfem [lattice gfem [ddion gfem [ ddlattice [ddionlattice] ]]]]]]]]]]] "
        self.interpreterQuickHelp = "Set the discretization type and the equations to be solved"
        self.interpreterLongHelp = "Set the discretization and the equations to be solved <> drift diffusion gfem solves the isothermal drift diffusion equation using the FEM-SUPG discretization. <> drift diffusion cvfem solves the isothermal drift diffusion equations using CVFEM-SG.<> drift diffusion effpg solves the isothermal drift diffusion equations using FEM-EFFPG. <> nlp or nlp gfem solves the nonlinear Poisson equation using FEM-SUPG. <> nlp cvfem solves the nonlinear Poisson equation using CVFEM-SG. <> laplace gfem solves the Laplace equation using FEM-SUPG. <> laplace cvfem solves the Laplace equation using CVFEM-SG. <> lattice gfem solves the lattice heat equation using FEM-SUPG. <> ddion gfem solves the isothermal memristor equations using FEM-SUPG. <> ddlattice solves the non-isothermal drift diffusion equations which discretization is specified using the discretization method line. <> ddionlattice solves the non-isothermal memristor model equations which discretization is specified using the discretization method line.  "

        # Register the xml required lines
        self.xmlRequiredLines = []
        self.xmlRequiredLinePriority = []
        self.xmlNewRequiredLines = []

        # Register the xml optional lines
        self.xmlOptionalLines = [[]]
        self.xmlOptionalLinePriority = [[]]
        self.xmlOptionalLines[0].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS,Type,string,Drift Diffusion")
        self.xmlOptionalLines[0].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS,Basis Type,string,HGrad")
        self.xmlOptionalLines[0].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS,Basis Order,int,1")
        self.xmlOptionalLines[0].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS,Integration Order,int,2")
        self.xmlOptionalLines[0].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS->Options,Solve Electron,string,True")
        self.xmlOptionalLines[0].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS->Options,Solve Hole,string,True")
        self.xmlOptionalLines[0].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS->Options,SUPG Stabilization,string,On")
        self.xmlOptionalLines[0].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS->Options,Tau_E,string,Tanh")
        self.xmlOptionalLines[0].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS->Options,Tau_H,string,Tanh")
        self.xmlOptionalLines[0].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS->Options,Length Scale,string,Stream")
        self.xmlOptionalLines.append([])
        self.xmlOptionalLines[1].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS,Type,string,EFFPG Drift Diffusion")
        self.xmlOptionalLines[1].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS,Basis Type,string,HGrad")
        self.xmlOptionalLines[1].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS,Basis Order,int,1")
        self.xmlOptionalLines[1].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS,Integration Order,int,2")
        self.xmlOptionalLines[1].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS->Options,Solve Electron,string,True")
        self.xmlOptionalLines[1].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS->Options,Solve Hole,string,True")
        self.xmlOptionalLines.append([])
        self.xmlOptionalLines[2].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS,Type,string,NLPoisson")
        self.xmlOptionalLines[2].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS,Basis Type,string,HGrad")
        self.xmlOptionalLines[2].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS,Basis Order,int,1")
        self.xmlOptionalLines[2].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS,Integration Order,int,2")
        self.xmlOptionalLines.append([])
        self.xmlOptionalLines[3].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS,Type,string,NLPoisson")
        self.xmlOptionalLines[3].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS,Basis Type,string,HGrad")
        self.xmlOptionalLines[3].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS,Basis Order,int,1")
        self.xmlOptionalLines[3].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS,Integration Order,int,2")
        self.xmlOptionalLines.append([])
        self.xmlOptionalLines[4].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS,Type,string,SGCVFEM NLPoisson")
        self.xmlOptionalLines[4].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS,Basis Type,string,HGrad")
        self.xmlOptionalLines[4].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS,Basis Order,int,1")
        self.xmlOptionalLines[4].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS,Integration Order,int,2")
        self.xmlOptionalLines.append([])
        self.xmlOptionalLines[5].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS,Type,string,SGCVFEM Drift Diffusion")
        self.xmlOptionalLines[5].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS,Basis Type,string,HGrad")
        self.xmlOptionalLines[5].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS,Basis Order,int,1")
        self.xmlOptionalLines[5].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS,Integration Order,int,2")
        self.xmlOptionalLines[5].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS->Options,Solve Electron,string,True")
        self.xmlOptionalLines[5].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS->Options,Solve Hole,string,True")
        self.xmlOptionalLines.append([])
        self.xmlOptionalLines[6].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS,Type,string,Laplace")
        self.xmlOptionalLines[6].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS,Basis Type,string,HGrad")
        self.xmlOptionalLines[6].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS,Basis Order,int,1")
        self.xmlOptionalLines[6].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS,Integration Order,int,2")
        self.xmlOptionalLines.append([])
        self.xmlOptionalLines[7].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS,Type,string,SGCVFEM Laplace")
        self.xmlOptionalLines[7].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS,Basis Type,string,HGrad")
        self.xmlOptionalLines[7].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS,Basis Order,int,1")
        self.xmlOptionalLines[7].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS,Integration Order,int,2")
        self.xmlOptionalLines.append([])
        self.xmlOptionalLines[8].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS,Type,string,Lattice")
        self.xmlOptionalLines[8].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS,Basis Type,string,HGrad")
        self.xmlOptionalLines[8].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS,Basis Order,int,1")
        self.xmlOptionalLines[8].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS,Integration Order,int,2")
        self.xmlOptionalLines[8].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS->Options,Heat Generation,string,Analytic")
        self.xmlOptionalLines.append([])
        self.xmlOptionalLines[9].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS,Type,string,DDIon")
        self.xmlOptionalLines[9].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS,Basis Type,string,HGrad")
        self.xmlOptionalLines[9].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS,Basis Order,int,1")
        self.xmlOptionalLines[9].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS,Integration Order,int,2")
        self.xmlOptionalLines[9].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS->Options,Solve Electron,string,True")
        self.xmlOptionalLines[9].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS->Options,Solve Hole,string,True")
        self.xmlOptionalLines[9].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS->Options,SUPG Stabilization,string,On")
        self.xmlOptionalLines[9].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS->Options,Tau_E,string,Tanh")
        self.xmlOptionalLines[9].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS->Options,Tau_H,string,Tanh")
        self.xmlOptionalLines[9].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS->Options,Length Scale,string,Stream")
        self.xmlOptionalLines[9].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS->Options,Add Source Term,string,Off")
        self.xmlOptionalLines.append([])
        self.xmlOptionalLines[10].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS,Type,string,DDLattice")
        self.xmlOptionalLines[10].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS,Basis Type,string,HGrad")
        self.xmlOptionalLines[10].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS,Basis Order,int,1")
        self.xmlOptionalLines[10].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS,Integration Order,int,2")
        self.xmlOptionalLines[10].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS->Options,Solve Electron,string,True")
        self.xmlOptionalLines[10].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS->Options,Solve Hole,string,True")
        self.xmlOptionalLines.append([])
        self.xmlOptionalLines[11].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS,Type,string,DDIonLattice")
        self.xmlOptionalLines[11].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS,Basis Type,string,HGrad")
        self.xmlOptionalLines[11].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS,Basis Order,int,1")
        self.xmlOptionalLines[11].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS,Integration Order,int,2")
        self.xmlOptionalLines[11].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS->Options,Solve Electron,string,True")
        self.xmlOptionalLines[11].append(" Charon->Physics Blocks->{physicsBlockName}->ANONYMOUS->Options,Solve Hole,string,True")
        self.xmlOptionalLinePriority[0].append(2)
        self.xmlOptionalLinePriority[0].append(2)
        self.xmlOptionalLinePriority[0].append(2)
        self.xmlOptionalLinePriority[0].append(2)
        self.xmlOptionalLinePriority[0].append(2)
        self.xmlOptionalLinePriority[0].append(2)
        self.xmlOptionalLinePriority[0].append(2)
        self.xmlOptionalLinePriority[0].append(2)
        self.xmlOptionalLinePriority[0].append(2)
        self.xmlOptionalLinePriority[0].append(2)
        self.xmlOptionalLinePriority.append([])
        self.xmlOptionalLinePriority[1].append(2)
        self.xmlOptionalLinePriority[1].append(2)
        self.xmlOptionalLinePriority[1].append(2)
        self.xmlOptionalLinePriority[1].append(2)
        self.xmlOptionalLinePriority[1].append(2)
        self.xmlOptionalLinePriority[1].append(2)
        self.xmlOptionalLinePriority.append([])
        self.xmlOptionalLinePriority[2].append(2)
        self.xmlOptionalLinePriority[2].append(2)
        self.xmlOptionalLinePriority[2].append(2)
        self.xmlOptionalLinePriority[2].append(2)
        self.xmlOptionalLinePriority.append([])
        self.xmlOptionalLinePriority[3].append(2)
        self.xmlOptionalLinePriority[3].append(2)
        self.xmlOptionalLinePriority[3].append(2)
        self.xmlOptionalLinePriority[3].append(2)
        self.xmlOptionalLinePriority.append([])
        self.xmlOptionalLinePriority[4].append(2)
        self.xmlOptionalLinePriority[4].append(2)
        self.xmlOptionalLinePriority[4].append(2)
        self.xmlOptionalLinePriority[4].append(2)
        self.xmlOptionalLinePriority.append([])
        self.xmlOptionalLinePriority[5].append(2)
        self.xmlOptionalLinePriority[5].append(2)
        self.xmlOptionalLinePriority[5].append(2)
        self.xmlOptionalLinePriority[5].append(2)
        self.xmlOptionalLinePriority[5].append(2)
        self.xmlOptionalLinePriority[5].append(2)
        self.xmlOptionalLinePriority.append([])
        self.xmlOptionalLinePriority[6].append(2)
        self.xmlOptionalLinePriority[6].append(2)
        self.xmlOptionalLinePriority[6].append(2)
        self.xmlOptionalLinePriority[6].append(2)
        self.xmlOptionalLinePriority.append([])
        self.xmlOptionalLinePriority[7].append(2)
        self.xmlOptionalLinePriority[7].append(2)
        self.xmlOptionalLinePriority[7].append(2)
        self.xmlOptionalLinePriority[7].append(2)
        self.xmlOptionalLinePriority.append([])
        self.xmlOptionalLinePriority[8].append(2)
        self.xmlOptionalLinePriority[8].append(2)
        self.xmlOptionalLinePriority[8].append(2)
        self.xmlOptionalLinePriority[8].append(2)
        self.xmlOptionalLinePriority[8].append(2)
        self.xmlOptionalLinePriority.append([])
        self.xmlOptionalLinePriority[9].append(2)
        self.xmlOptionalLinePriority[9].append(2)
        self.xmlOptionalLinePriority[9].append(2)
        self.xmlOptionalLinePriority[9].append(2)
        self.xmlOptionalLinePriority[9].append(2)
        self.xmlOptionalLinePriority[9].append(2)
        self.xmlOptionalLinePriority[9].append(2)
        self.xmlOptionalLinePriority[9].append(2)
        self.xmlOptionalLinePriority[9].append(2)
        self.xmlOptionalLinePriority[9].append(2)
        self.xmlOptionalLinePriority[9].append(2)
        self.xmlOptionalLinePriority.append([])
        self.xmlOptionalLinePriority[10].append(2)
        self.xmlOptionalLinePriority[10].append(2)
        self.xmlOptionalLinePriority[10].append(2)
        self.xmlOptionalLinePriority[10].append(2)
        self.xmlOptionalLinePriority[10].append(2)
        self.xmlOptionalLinePriority[10].append(2)
        self.xmlOptionalLinePriority.append([])
        self.xmlOptionalLinePriority[11].append(2)
        self.xmlOptionalLinePriority[11].append(2)
        self.xmlOptionalLinePriority[11].append(2)
        self.xmlOptionalLinePriority[11].append(2)
        self.xmlOptionalLinePriority[11].append(2)
        self.xmlOptionalLinePriority[11].append(2)

        # Register the xml default lines
        self.xmlDefaultLines = []
        self.xmlDefaultLinePriority = []

        self.xmlReturned = []
        self.xmlPriorityCode = []
        self.specialInformation = []



    def isThisMe(self,tokenizer,line):
        affirmPrimary = False
        for index,fO in enumerate(self.foundOptions):
            self.foundOptions[index] = False
        self.primaryVariables = {}
        self.optionVariables = []*len(self.options)
        (affirmPrimary,self.foundOptions,self.primaryVariables,self.optionVariables,suggestedSyntax) = tokenizer.isItMe(line,self.primary,self.options)
        return (affirmPrimary,suggestedSyntax)



    def getName(self):
        # Return parser name
         return self.parserName



    def getHelp(self,verbosity):
        # Return help content
        if verbosity.lower() == "long":
            return (self.interpreterHelpLine,self.interpreterLongHelp)
        else:
            return (self.interpreterHelpLine,self.interpreterQuickHelp)



    def generateXML(self,tokenizer,line):
        # Tokenize the line
        lineTokens = tokenizer.tokenize(line)
        self.xmlNewRequiredLines[:] = []
        for xL in self.xmlRequiredLines:
            self.xmlNewRequiredLines.append(xL)

        for key in self.primaryVariables:
            for nRLIndex,nRL in enumerate(self.xmlNewRequiredLines):
                self.xmlNewRequiredLines[nRLIndex] = nRL.replace(key,self.primaryVariables[key])
            for siIndex, siValue in enumerate(self.specialInformation):
                self.specialInformation[siIndex] = [item.replace(key,self.primaryVariables[key]) for item in siValue]

        for index,xmlLine in enumerate(self.xmlNewRequiredLines):
            self.xmlReturned.append(xmlLine)
            self.xmlPriorityCode.append(self.xmlRequiredLinePriority[index]) #required lines have priority code 2
        for optIndex,fO in enumerate(self.foundOptions):
            if fO == True:
                self.Returned=copy.deepcopy(self.xmlOptionalLines[optIndex])
                for key in self.primaryVariables:
                    for retIndex,xOL in enumerate(self.Returned):
                        self.Returned[retIndex] = xOL.replace(key,self.primaryVariables[key])
                for key in self.optionVariables[optIndex]:
                    for retIndex,xOL in enumerate(self.Returned):
                        self.Returned[retIndex] = xOL.replace(key,self.optionVariables[optIndex][key])
                        for siIndex, siValue in enumerate(self.specialInformation):
                            self.specialInformation[siIndex] = [item.replace(key,self.optionVariables[optIndex][key]) for item in siValue]
                for retIndex,ret in enumerate(self.Returned):
                    self.xmlReturned.append(ret)
                    self.xmlPriorityCode.append(self.xmlOptionalLinePriority[optIndex][retIndex])

        for xmlLine in self.xmlDefaultLines:
            self.xmlReturned.append(xmlLine)
            self.xmlPriorityCode.append(1) #optional lines have priority code 1

        return (self.xmlReturned,self.xmlPriorityCode,self.specialInformation)



