
try:
    import coloramaDISABLED as colors
except ImportError:
    class stubColors:
        "subs for colors when colors doesn't exist on system"
    
        def __init__(self):
            self.Fore = colorClass()
            self.Back = colorClass()
            self.Style = styleClass()
    
    class colorClass():
        "stubbed color class"
    
        def __init__(self):
            self.BLACK = ""
            self.BLUE = ""
            self.WHITE = ""
            self.RED = ""
            self.GREEN = ""
    
    class styleClass():
        "stubbed style class"
    
        def __init__(self):
            self.RESET_ALL = ""
    colors = stubColors()

import sys
from .charonLineParserSolverPack5 import *
from .charonLineParserNoxSolver import *
from .charonLineParserSolverPack2 import *
from .charonLineParserLinearSolverType import *
from .charonLineParserSolverPack3 import *
from .charonLineParserTimeIntegratorMinTimeStepSize import *
from .charonLineParserTimeIntegratorMaxTimeStepSize import *
from .charonLineParserTimeIntegratorRelativeTolerance import *
from .charonLineParserSolverTolerance import *
from .charonLineParserTimeIntegratorFinalTime import *
from .charonLineParserAztecTolerance import *
from .charonLineParserThyraFunctionScaling import *
from .charonLineParserAztecMaxIterations import *
from .charonLineParserLineSearchMethod import *
from .charonLineParserTimeIntegratorFixedStepSize import *
from .charonLineParserSolverPack7 import *
from .charonLineParserSolverPackHB import *
from .charonLineParserTransientSolverPack2 import *
from .charonLineParserMaxNonlinearIterations import *
from .charonLineParserSolverPack6 import *
from .charonLineParserGMRESILU import *
from .charonLineParserAztecKrylovSubSize import *
from .charonLineParserIfpackReorderingType import *
from .charonLineParserSolverPack4 import *
from .charonLineParserTimeIntegratorInitialStepSize import *
from .charonLineParserIfpackLevelOfFill import *
from .charonLineParserTransientSolverPack1 import *
from .charonLineParserTimeIntegratorVariableStepSize import *
from .charonLineParserTimeIntegratorAbsoluteTolerance import *
from .charonLineParserSolverPack1 import *
from .charonBlockParserWRMSParameters import *
from .charonBlockParserTpetraSettingsBlock import *
from .WRMSParameters.WRMSParametersParserLib import *
from .TpetraSettingsBlock.TpetraSettingsBlockParserLib import *



class SolverBlockParserLib:
    "This is the  SolverBlockParserLib parser library "


    def __init__(self):
        # set the parser library name 
        self.parserLibName = "SolverBlockParserLib"
        # create the linparser objects 
        self.lineParsers = []
        self.lineParsers.append(charonLineParserSolverPack5())
        self.lineParsers.append(charonLineParserNoxSolver())
        self.lineParsers.append(charonLineParserSolverPack2())
        self.lineParsers.append(charonLineParserLinearSolverType())
        self.lineParsers.append(charonLineParserSolverPack3())
        self.lineParsers.append(charonLineParserTimeIntegratorMinTimeStepSize())
        self.lineParsers.append(charonLineParserTimeIntegratorMaxTimeStepSize())
        self.lineParsers.append(charonLineParserTimeIntegratorRelativeTolerance())
        self.lineParsers.append(charonLineParserSolverTolerance())
        self.lineParsers.append(charonLineParserTimeIntegratorFinalTime())
        self.lineParsers.append(charonLineParserAztecTolerance())
        self.lineParsers.append(charonLineParserThyraFunctionScaling())
        self.lineParsers.append(charonLineParserAztecMaxIterations())
        self.lineParsers.append(charonLineParserLineSearchMethod())
        self.lineParsers.append(charonLineParserTimeIntegratorFixedStepSize())
        self.lineParsers.append(charonLineParserSolverPack7())
        self.lineParsers.append(charonLineParserSolverPackHB())
        self.lineParsers.append(charonLineParserTransientSolverPack2())
        self.lineParsers.append(charonLineParserMaxNonlinearIterations())
        self.lineParsers.append(charonLineParserSolverPack6())
        self.lineParsers.append(charonLineParserGMRESILU())
        self.lineParsers.append(charonLineParserAztecKrylovSubSize())
        self.lineParsers.append(charonLineParserIfpackReorderingType())
        self.lineParsers.append(charonLineParserSolverPack4())
        self.lineParsers.append(charonLineParserTimeIntegratorInitialStepSize())
        self.lineParsers.append(charonLineParserIfpackLevelOfFill())
        self.lineParsers.append(charonLineParserTransientSolverPack1())
        self.lineParsers.append(charonLineParserTimeIntegratorVariableStepSize())
        self.lineParsers.append(charonLineParserTimeIntegratorAbsoluteTolerance())
        self.lineParsers.append(charonLineParserSolverPack1())
        # create the blockparser objects 
        self.blockParsers = []
        self.blockParsers.append([charonBlockParserWRMSParameters(),WRMSParametersParserLib()])
        self.blockParsers.append([charonBlockParserTpetraSettingsBlock(),TpetraSettingsBlockParserLib()])
        # create the parserLibrary objects 
        parserLibraries = []
        parserLibraries.append(WRMSParametersParserLib())
        parserLibraries.append(TpetraSettingsBlockParserLib())


    def isThisMyLine(self,tokenizer,line):
        suggestedSyntax = []
        for lP in self.lineParsers:
            (self.isThisMe,suggestedSyntaxItem) = lP.isThisMe(tokenizer,line)
            if suggestedSyntaxItem is not "":
                suggestedSyntax.append(suggestedSyntaxItem)
            if self.isThisMe == True:
                return (True,lP,suggestedSyntax)
        return (False,None,suggestedSyntax)


    def isThisMyBlock(self,tokenizer,line):
        for bP in self.blockParsers:
            self.isThisMe = bP[0].isThisMe(tokenizer,line)
            if self.isThisMe == True:
                return (True,bP[0],bP[1])
        return (False,None,None)


    def generateHelp(self,genHelp,indent):
        self.addIndent = "     "
        cRStyle = ""
        for lP in self.lineParsers:
            (self.helpLine,self.helpContent) = lP.getHelp(genHelp)
            self.helpContentList = self.helpContent.split("<>")
            print (cRStyle+indent+colors.Fore.RED+colors.Back.WHITE+self.helpLine)
            cRStyle = "\n"
            for hCL in self.helpContentList:
                print ("\t"+indent+colors.Fore.BLUE+colors.Back.WHITE+hCL.lstrip())
        for bP in range(len(self.blockParsers)):
            print (indent+colors.Fore.GREEN+colors.Back.WHITE+self.blockParsers[bP][0].getHelpLine().lstrip())
            self.blockParsers[bP][1].generateHelp(genHelp,indent+self.addIndent)
            print (indent+colors.Fore.GREEN+colors.Back.WHITE+self.blockParsers[bP][0].getHelpLine().replace("start","end").lstrip())
            print (indent+colors.Style.RESET_ALL)


    def getName(self):
        return self.parserLibName
