
try:
    import coloramaDISABLED as colors
except ImportError:
    class stubColors:
        "subs for colors when colors doesn't exist on system"
    
        def __init__(self):
            self.Fore = colorClass()
            self.Back = colorClass()
            self.Style = styleClass()
    
    class colorClass():
        "stubbed color class"
    
        def __init__(self):
            self.BLACK = ""
            self.BLUE = ""
            self.WHITE = ""
            self.RED = ""
            self.GREEN = ""
    
    class styleClass():
        "stubbed style class"
    
        def __init__(self):
            self.RESET_ALL = ""
    colors = stubColors()

import sys
from .charonLineParserLinearSolverTolerance import *
from .charonLineParserTransientMazStepSize import *
from .charonLineParserTransientBasicVsEtaMin import *
from .charonLineParserTransientDecreaseFactor import *
from .charonLineParserNonlinearFNorm import *
from .charonLineParserTransientInitialStepSize import *
from .charonLineParserTransientRelativeTolerance import *
from .charonLineParserTransientBasicVsEtaMax import *
from .charonLineParserTransientTimeStepping import *
from .charonLineParserTransientIncreaseFactor import *
from .charonLineParserPrecType import *
from .charonLineParserTransientFinalTime import *
from .charonLineParserNonLinearIterCount import *
from .charonLineParserTransientMinStepSize import *
from .charonLineParserProblemType import *
from .charonLineParserVerbosityLevel import *
from .charonLineParserTransientNumOfFirstSteps import *
from .charonLineParserLinearSolver import *
from .charonLineParserTransientAbsoluteTolerance import *
from .charonBlockParserAztecSolverBlock import *
from .charonBlockParserNonlinearSolverWRMSBlock import *
from .AztecSolverBlock.AztecSolverBlockParserLib import *
from .NonlinearSolverWRMSBlock.NonlinearSolverWRMSBlockParserLib import *



class TpetraSettingsBlockParserLib:
    "This is the  TpetraSettingsBlockParserLib parser library "


    def __init__(self):
        # set the parser library name 
        self.parserLibName = "TpetraSettingsBlockParserLib"
        # create the linparser objects 
        self.lineParsers = []
        self.lineParsers.append(charonLineParserLinearSolverTolerance())
        self.lineParsers.append(charonLineParserTransientMazStepSize())
        self.lineParsers.append(charonLineParserTransientBasicVsEtaMin())
        self.lineParsers.append(charonLineParserTransientDecreaseFactor())
        self.lineParsers.append(charonLineParserNonlinearFNorm())
        self.lineParsers.append(charonLineParserTransientInitialStepSize())
        self.lineParsers.append(charonLineParserTransientRelativeTolerance())
        self.lineParsers.append(charonLineParserTransientBasicVsEtaMax())
        self.lineParsers.append(charonLineParserTransientTimeStepping())
        self.lineParsers.append(charonLineParserTransientIncreaseFactor())
        self.lineParsers.append(charonLineParserPrecType())
        self.lineParsers.append(charonLineParserTransientFinalTime())
        self.lineParsers.append(charonLineParserNonLinearIterCount())
        self.lineParsers.append(charonLineParserTransientMinStepSize())
        self.lineParsers.append(charonLineParserProblemType())
        self.lineParsers.append(charonLineParserVerbosityLevel())
        self.lineParsers.append(charonLineParserTransientNumOfFirstSteps())
        self.lineParsers.append(charonLineParserLinearSolver())
        self.lineParsers.append(charonLineParserTransientAbsoluteTolerance())
        # create the blockparser objects 
        self.blockParsers = []
        self.blockParsers.append([charonBlockParserAztecSolverBlock(),AztecSolverBlockParserLib()])
        self.blockParsers.append([charonBlockParserNonlinearSolverWRMSBlock(),NonlinearSolverWRMSBlockParserLib()])
        # create the parserLibrary objects 
        parserLibraries = []
        parserLibraries.append(AztecSolverBlockParserLib())
        parserLibraries.append(NonlinearSolverWRMSBlockParserLib())


    def isThisMyLine(self,tokenizer,line):
        suggestedSyntax = []
        for lP in self.lineParsers:
            (self.isThisMe,suggestedSyntaxItem) = lP.isThisMe(tokenizer,line)
            if suggestedSyntaxItem is not "":
                suggestedSyntax.append(suggestedSyntaxItem)
            if self.isThisMe == True:
                return (True,lP,suggestedSyntax)
        return (False,None,suggestedSyntax)


    def isThisMyBlock(self,tokenizer,line):
        for bP in self.blockParsers:
            self.isThisMe = bP[0].isThisMe(tokenizer,line)
            if self.isThisMe == True:
                return (True,bP[0],bP[1])
        return (False,None,None)


    def generateHelp(self,genHelp,indent):
        self.addIndent = "     "
        cRStyle = ""
        for lP in self.lineParsers:
            (self.helpLine,self.helpContent) = lP.getHelp(genHelp)
            self.helpContentList = self.helpContent.split("<>")
            print (cRStyle+indent+colors.Fore.RED+colors.Back.WHITE+self.helpLine)
            cRStyle = "\n"
            for hCL in self.helpContentList:
                print ("\t"+indent+colors.Fore.BLUE+colors.Back.WHITE+hCL.lstrip())
        for bP in range(len(self.blockParsers)):
            print (indent+colors.Fore.GREEN+colors.Back.WHITE+self.blockParsers[bP][0].getHelpLine().lstrip())
            self.blockParsers[bP][1].generateHelp(genHelp,indent+self.addIndent)
            print (indent+colors.Fore.GREEN+colors.Back.WHITE+self.blockParsers[bP][0].getHelpLine().replace("start","end").lstrip())
            print (indent+colors.Style.RESET_ALL)


    def getName(self):
        return self.parserLibName
