
import copy


class charonLineParserProblemType:
    "ProblemType parser"

    def __init__(self):
        # Register the parsing keys
        self.parserName = "ProblemType"
        self.primary = "problem type is"
        self.options = []
        self.options.append("steady state")
        self.options.append("constrained steady state")
        self.options.append("householder constrained steady state")
        self.options.append("parameter sweep")
        self.options.append("constrained parameter sweep")
        self.options.append("householder constrained parameter sweep")
        self.options.append("transient")
        self.options.append("tempus")
        self.options.append("harmonic balance")
        self.options.append("harmonic balance sweep")
        self.foundOptions = []
        for index,opt in enumerate(self.options):
            self.foundOptions.append(False)
        self.primaryVariables = []
        self.optionVariables = [[]]*len(self.options)
        self.interpreterHelpLine = "problem type is [ steady state [ constrained steady state [ householder constrained steady state [ parameter sweep [ constrained parameter sweep [ householder constrained parameter sweep [ transient [ tempus [harmonic balance [harmonic balance sweep ]]]]]]]]]] "
        self.interpreterQuickHelp = "Use typical settings for Tpetra and the specified problem type"
        self.interpreterLongHelp = "Use typical settings for Tpetra and the specified problem type"

        # Register the xml required lines
        self.xmlRequiredLines = []
        self.xmlRequiredLinePriority = []
        self.xmlRequiredLines.append("Charon->Solution Control,Compute Sensitivities,bool,false")
        self.xmlRequiredLines.append("Charon->Solution Control,Jacobian Operator,string,Have Jacobian")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Direction,Method,string,Newton")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Direction->Newton,Forcing Term Method,string,Constant")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Direction->Newton,Rescue Bad Newton Solve,bool,true")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos->Linear Solver Types->Belos,Solver Type,string,Block GMRES")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos->Linear Solver Types->Belos->VerboseObject,Verbosity Level,string,none")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos->Linear Solver Types->Belos->Solver Types->Block GMRES,Convergence Tolerance,double,1.0e-06")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos->Linear Solver Types->Belos->Solver Types->Block GMRES,Output Frequency,int,20")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos->Linear Solver Types->Belos->Solver Types->Block GMRES,Output Style,int,1")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos->Linear Solver Types->Belos->Solver Types->Block GMRES,Verbosity,int,33")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos->Linear Solver Types->Belos->Solver Types->Block GMRES,Block Size,int,1")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos->Linear Solver Types->Belos->Solver Types->Block GMRES,Num Blocks,int,100")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos->Linear Solver Types->Belos->Solver Types->Block GMRES,Maximum Iterations,int,300")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos->Linear Solver Types->Belos->Solver Types->Block GMRES,Flexible Gmres,bool,false")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos->Preconditioner Types->Teko,Inverse Type,string,GS")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos->Preconditioner Types->Teko,Write Block Operator,bool,false")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos->Preconditioner Types->Teko,Test Block Operator,bool,false")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos->Preconditioner Types->Teko->Inverse Factory Library->GS,Type,string,Block Gauss-Seidel")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos->Preconditioner Types->Teko->Inverse Factory Library->GS,Use Upper Triangle,bool,true")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos->Preconditioner Types->Teko->Inverse Factory Library->GS,Inverse Type 1,string,ILU_tpetra")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos->Preconditioner Types->Teko->Inverse Factory Library->GS,Inverse Type 2,string,Id")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos->Preconditioner Types->Teko->Inverse Factory Library->ILU_tpetra,Type,string,Ifpack2")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos->Preconditioner Types->Teko->Inverse Factory Library->ILU_tpetra,Overlap,int,1")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos->Preconditioner Types->Teko->Inverse Factory Library->ILU_tpetra->Ifpack2 Settings,fact: drop tolerance,double,0.0")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos->Preconditioner Types->Teko->Inverse Factory Library->ILU_tpetra->Ifpack2 Settings,fact: ilut level-of-fill,double,1.0")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos->Preconditioner Types->Teko->Inverse Factory Library->Id,Type,string,Identity")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos->Linear Solver Types->AztecOO->Forward Solve->AztecOO Settings,Aztec Solver,string,GMRES")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos->Linear Solver Types->AztecOO->Forward Solve->AztecOO Settings,Convergence Test,string,r0")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos->Linear Solver Types->AztecOO->Forward Solve->AztecOO Settings,Output Frequency,int,20")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos->Linear Solver Types->AztecOO->Forward Solve,Max Iterations,int,300")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos->Preconditioner Types->Ifpack2,Prec Type,string,ILUT")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos->Preconditioner Types->Ifpack2->Ifpack2 Settings,schwarz: overlap level,int,1")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos->Preconditioner Types->Ifpack2->Ifpack2 Settings,schwarz: use reordering,bool,false")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos->Preconditioner Types->Ifpack2->Ifpack2 Settings,schwarz: combine mode,string,add")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos->Preconditioner Types->Ifpack2->Ifpack2 Settings,fact: ilut level-of-fill,double,3.0")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos->Preconditioner Types->Ifpack2->Ifpack2 Settings,fact: iluk level-of-fill,int,3")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos->Preconditioner Types->Ifpack2->Ifpack2 Settings,fact: iluk level-of-overlap,int,1")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX,Nonlinear Solver,string,Line Search Based")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Thyra Group Options,Function Scaling,string,Row Sum")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Line Search->Full Step,Full Step,double,1")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Line Search,Method,string,Full Step")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Printing,Output Precision,int,3")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Printing,Output Processor,int,0")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Printing->Output Information,Error,bool,true")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Printing->Output Information,Warning,bool,true")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Printing->Output Information,Inner Iteration,bool,true")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Printing->Output Information,Outer Iteration,bool,true")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Printing->Output Information,Outer Iteration StatusTest,bool,true")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Printing->Output Information,Parameters,bool,true")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Printing->Output Information,Details,bool,true")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Printing->Output Information,Linear Solver Details,bool,true")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Printing->Output Information,Stepper Iteration,bool,true")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Printing->Output Information,Stepper Details,bool,true")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Printing->Output Information,Stepper Parameters,bool,true")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Solver Options,Status Test Check Type,string,Minimal")
        self.xmlRequiredLines.append("Charon->Solution Control->Tempus->Default Stepper->Default Solver->NOX,Nonlinear Solver,string,Line Search Based")
        self.xmlRequiredLines.append("Charon->Solution Control->Tempus->Default Stepper->Default Solver->NOX->Thyra Group Options,Function Scaling,string,Row Sum")
        self.xmlRequiredLines.append("Charon->Solution Control->Tempus->Default Stepper->Default Solver->NOX->Line Search->Full Step,Full Step,double,1")
        self.xmlRequiredLines.append("Charon->Solution Control->Tempus->Default Stepper->Default Solver->NOX->Line Search,Method,string,Full Step")
        self.xmlRequiredLines.append("Charon->Solution Control->Tempus->Default Stepper->Default Solver->NOX->Printing,Output Precision,int,3")
        self.xmlRequiredLines.append("Charon->Solution Control->Tempus->Default Stepper->Default Solver->NOX->Printing,Output Processor,int,0")
        self.xmlRequiredLines.append("Charon->Solution Control->Tempus->Default Stepper->Default Solver->NOX->Printing->Output Information,Error,bool,true")
        self.xmlRequiredLines.append("Charon->Solution Control->Tempus->Default Stepper->Default Solver->NOX->Printing->Output Information,Warning,bool,true")
        self.xmlRequiredLines.append("Charon->Solution Control->Tempus->Default Stepper->Default Solver->NOX->Printing->Output Information,Inner Iteration,bool,true")
        self.xmlRequiredLines.append("Charon->Solution Control->Tempus->Default Stepper->Default Solver->NOX->Printing->Output Information,Outer Iteration,bool,true")
        self.xmlRequiredLines.append("Charon->Solution Control->Tempus->Default Stepper->Default Solver->NOX->Printing->Output Information,Outer Iteration StatusTest,bool,true")
        self.xmlRequiredLines.append("Charon->Solution Control->Tempus->Default Stepper->Default Solver->NOX->Printing->Output Information,Parameters,bool,true")
        self.xmlRequiredLines.append("Charon->Solution Control->Tempus->Default Stepper->Default Solver->NOX->Printing->Output Information,Details,bool,true")
        self.xmlRequiredLines.append("Charon->Solution Control->Tempus->Default Stepper->Default Solver->NOX->Printing->Output Information,Linear Solver Details,bool,true")
        self.xmlRequiredLines.append("Charon->Solution Control->Tempus->Default Stepper->Default Solver->NOX->Printing->Output Information,Stepper Iteration,bool,true")
        self.xmlRequiredLines.append("Charon->Solution Control->Tempus->Default Stepper->Default Solver->NOX->Printing->Output Information,Stepper Details,bool,true")
        self.xmlRequiredLines.append("Charon->Solution Control->Tempus->Default Stepper->Default Solver->NOX->Printing->Output Information,Stepper Parameters,bool,true")
        self.xmlRequiredLines.append("Charon->Solution Control->Tempus->Default Stepper->Default Solver->NOX->Solver Options,Status Test Check Type,string,Minimal")
        self.xmlRequiredLines.append("Charon->Solution Control->Rythmos,Final Time,double,5.5e-4")
        self.xmlRequiredLines.append("Charon->Solution Control->Rythmos->Rythmos Integration Control,Take Variable Steps,bool,true")
        self.xmlRequiredLines.append("Charon->Solution Control->Rythmos,Stepper Type,string,Backward Euler")
        self.xmlRequiredLines.append("Charon->Solution Control->Rythmos->Rythmos Step Control Strategy,Initial Step Size,double,1e-5")
        self.xmlRequiredLines.append("Charon->Solution Control->Rythmos->Rythmos Step Control Strategy,Min Step Size,double,1e-7")
        self.xmlRequiredLines.append("Charon->Solution Control->Rythmos->Rythmos Step Control Strategy,Max Step Size,double,1.0e-4")
        self.xmlRequiredLines.append("Charon->Solution Control->Rythmos->Rythmos Step Control Strategy,Number of Constant First Order Steps,int,5")
        self.xmlRequiredLines.append("Charon->Solution Control->Rythmos->Rythmos Step Control Strategy,Step Size Decrease Factor,double,0.2")
        self.xmlRequiredLines.append("Charon->Solution Control->Rythmos->Rythmos Step Control Strategy,Step Size Increase Factor,double,1.5")
        self.xmlRequiredLines.append("Charon->Solution Control->Rythmos,Nonlinear Solver Type,string,NOX")
        self.xmlRequiredLines.append("Charon->Solution Control->Rythmos->Rythmos Stepper->VerboseObject,Verbosity Level,string,none")
        self.xmlRequiredLines.append("Charon->Solution Control->Rythmos->Rythmos Integrator->VerboseObject,Verbosity Level,string,none")
        self.xmlRequiredLines.append("Charon->Solution Control->Rythmos->Rythmos Step Control Strategy,Min Order,int,1")
        self.xmlRequiredLines.append("Charon->Solution Control->Rythmos->Rythmos Step Control Strategy,Max Order,int,1")
        self.xmlRequiredLines.append("Charon->Solution Control->Tempus,Integrator Name,string,Default Integrator")
        self.xmlRequiredLines.append("Charon->Solution Control->Tempus->Default Integrator,Integrator Type,string,Integrator Basic")
        self.xmlRequiredLines.append("Charon->Solution Control->Tempus->Default Integrator->Time Step Control,Output Index Interval,int,0")
        self.xmlRequiredLines.append("Charon->Solution Control->Tempus->Default Integrator->Time Step Control,Initial Time Step,double,1e-5")
        self.xmlRequiredLines.append("Charon->Solution Control->Tempus->Default Integrator,Stepper Name,string,Default Stepper")
        self.xmlRequiredLines.append("Charon->Solution Control->Tempus->Default Stepper,Stepper Type,string,Backward Euler")
        self.xmlRequiredLines.append("Charon->Solution Control->Tempus->Default Stepper,Solver Name,string,Default Solver")
        self.xmlRequiredLines.append("Charon->Solution Control->Tempus->Default Stepper->Default Solver->NOX,Nonlinear Solver,string,Line Search Based")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Status Tests,Test Type,string,Combo")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Status Tests,Combo Type,string,OR")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Status Tests,Number of Tests,int,3")
        self.xmlRequiredLines.append("Charon->Solution Control->Tempus->Default Stepper->Default Solver->NOX->Status Tests,Test Type,string,Combo")
        self.xmlRequiredLines.append("Charon->Solution Control->Tempus->Default Stepper->Default Solver->NOX->Status Tests,Combo Type,string,OR")
        self.xmlRequiredLines.append("Charon->Solution Control->Tempus->Default Stepper->Default Solver->NOX->Status Tests,Number of Tests,int,3")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Status Tests->Test 1,Test Type,string,MaxIters")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Status Tests->Test 2,Test Type,string,FiniteValue")
        self.xmlRequiredLines.append("Charon->Solution Control->NOX->Status Tests->Test 2,Vector Type,string,F Vector")
        self.xmlRequiredLines.append("Charon->Solution Control->Tempus->Default Stepper->Default Solver->NOX->Status Tests->Test 1,Test Type,string,MaxIters")
        self.xmlRequiredLines.append("Charon->Solution Control->Tempus->Default Stepper->Default Solver->NOX->Status Tests->Test 2,Test Type,string,FiniteValue")
        self.xmlRequiredLines.append("Charon->Solution Control->Tempus->Default Stepper->Default Solver->NOX->Status Tests->Test 2,Vector Type,string,F Vector")
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlNewRequiredLines = []

        # Register the xml optional lines
        self.xmlOptionalLines = [[]]
        self.xmlOptionalLinePriority = [[]]
        self.xmlOptionalLines[0].append(" Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos,Linear Solver Type,string,AztecOO")
        self.xmlOptionalLines[0].append(" Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos,Preconditioner Type,string,Ifpack2")
        self.xmlOptionalLines.append([])
        self.xmlOptionalLines[1].append(" Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos,Linear Solver Type,string,Belos")
        self.xmlOptionalLines[1].append(" Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos,Preconditioner Type,string,Teko")
        self.xmlOptionalLines.append([])
        self.xmlOptionalLines[2].append(" Charon->Solution Control,Piro Solver,string,NOX-Constrained")
        self.xmlOptionalLines[2].append(" Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos,Linear Solver Type,string,Belos")
        self.xmlOptionalLines[2].append(" Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos,Preconditioner Type,string,Ifpack2")
        self.xmlOptionalLines.append([])
        self.xmlOptionalLines[3].append(" Charon->Solution Control,Piro Solver,string,LOCA")
        self.xmlOptionalLines[3].append(" Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos,Linear Solver Type,string,AztecOO")
        self.xmlOptionalLines[3].append(" Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos,Preconditioner Type,string,Ifpack2")
        self.xmlOptionalLines.append([])
        self.xmlOptionalLines[4].append(" Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos,Linear Solver Type,string,Belos")
        self.xmlOptionalLines[4].append(" Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos,Preconditioner Type,string,Teko")
        self.xmlOptionalLines.append([])
        self.xmlOptionalLines[5].append(" Charon->Solution Control,Piro Solver,string,LOCA-Constrained")
        self.xmlOptionalLines[5].append(" Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos,Linear Solver Type,string,Belos")
        self.xmlOptionalLines[5].append(" Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos,Preconditioner Type,string,Ifpack2")
        self.xmlOptionalLines.append([])
        self.xmlOptionalLines[6].append(" Charon->Solution Control,Piro Solver,string,Rythmos")
        self.xmlOptionalLines[6].append(" Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos,Linear Solver Type,string,AztecOO")
        self.xmlOptionalLines[6].append(" Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos,Preconditioner Type,string,Ifpack2")
        self.xmlOptionalLines.append([])
        self.xmlOptionalLines[7].append(" Charon->Solution Control,Piro Solver,string,Tempus")
        self.xmlOptionalLines[7].append(" Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos,Linear Solver Type,string,AztecOO")
        self.xmlOptionalLines[7].append(" Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos,Preconditioner Type,string,Ifpack2")
        self.xmlOptionalLines.append([])
        self.xmlOptionalLines[8].append(" Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos,Linear Solver Type,string,AztecOO")
        self.xmlOptionalLines[8].append(" Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos,Preconditioner Type,string,Ifpack2")
        self.xmlOptionalLines[8].append(" Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos->Preconditioner Types->Ifpack2->Ifpack2 Settings,fact: absolute threshold,double,1e-3")
        self.xmlOptionalLines.append([])
        self.xmlOptionalLines[9].append(" Charon->Solution Control,Piro Solver,string,LOCA")
        self.xmlOptionalLines[9].append(" Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos,Linear Solver Type,string,AztecOO")
        self.xmlOptionalLines[9].append(" Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos,Preconditioner Type,string,Ifpack2")
        self.xmlOptionalLines[9].append(" Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos->Preconditioner Types->Ifpack2->Ifpack2 Settings,fact: absolute threshold,double,1e-3")
        self.xmlOptionalLinePriority[0].append(2)
        self.xmlOptionalLinePriority[0].append(2)
        self.xmlOptionalLinePriority.append([])
        self.xmlOptionalLinePriority[1].append(2)
        self.xmlOptionalLinePriority[1].append(2)
        self.xmlOptionalLinePriority.append([])
        self.xmlOptionalLinePriority[2].append(2)
        self.xmlOptionalLinePriority[2].append(2)
        self.xmlOptionalLinePriority[2].append(2)
        self.xmlOptionalLinePriority.append([])
        self.xmlOptionalLinePriority[3].append(2)
        self.xmlOptionalLinePriority[3].append(2)
        self.xmlOptionalLinePriority[3].append(2)
        self.xmlOptionalLinePriority.append([])
        self.xmlOptionalLinePriority[4].append(2)
        self.xmlOptionalLinePriority[4].append(2)
        self.xmlOptionalLinePriority.append([])
        self.xmlOptionalLinePriority[5].append(2)
        self.xmlOptionalLinePriority[5].append(2)
        self.xmlOptionalLinePriority[5].append(2)
        self.xmlOptionalLinePriority.append([])
        self.xmlOptionalLinePriority[6].append(2)
        self.xmlOptionalLinePriority[6].append(2)
        self.xmlOptionalLinePriority[6].append(2)
        self.xmlOptionalLinePriority.append([])
        self.xmlOptionalLinePriority[7].append(2)
        self.xmlOptionalLinePriority[7].append(2)
        self.xmlOptionalLinePriority[7].append(2)
        self.xmlOptionalLinePriority.append([])
        self.xmlOptionalLinePriority[8].append(2)
        self.xmlOptionalLinePriority[8].append(2)
        self.xmlOptionalLinePriority[8].append(2)
        self.xmlOptionalLinePriority.append([])
        self.xmlOptionalLinePriority[9].append(2)
        self.xmlOptionalLinePriority[9].append(2)
        self.xmlOptionalLinePriority[9].append(2)
        self.xmlOptionalLinePriority[9].append(2)

        # Register the xml default lines
        self.xmlDefaultLines = []
        self.xmlDefaultLinePriority = []
        self.xmlDefaultLines.append("Charon->Solution Control,Piro Solver,string,NOX")
        self.xmlDefaultLines.append("Charon->Solution Control->NOX->Direction->Newton->Linear Solver,Tolerance,double,1.0e-06")
        self.xmlDefaultLines.append("Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos->Linear Solver Types->AztecOO->Forward Solve->AztecOO Settings,Size of Krylov Subspace,int,200")
        self.xmlDefaultLines.append("Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos->Linear Solver Types->AztecOO->Forward Solve->AztecOO Settings,RCM Reordering,string,Disabled")
        self.xmlDefaultLines.append("Charon->Solution Control->NOX->Direction->Newton->Stratimikos Linear Solver->Stratimikos->Linear Solver Types->AztecOO->Forward Solve->AztecOO Settings,Overlap,int,0")
        self.xmlDefaultLines.append("Charon->Solution Control->NOX->Status Tests->Test 0,Test Type,string,Combo")
        self.xmlDefaultLines.append("Charon->Solution Control->NOX->Status Tests->Test 0,Combo Type,string,AND")
        self.xmlDefaultLines.append("Charon->Solution Control->NOX->Status Tests->Test 0,Number of Tests,int,2")
        self.xmlDefaultLines.append("Charon->Solution Control->NOX->Status Tests->Test 0->Test 0,Test Type,string,NormF")
        self.xmlDefaultLines.append("Charon->Solution Control->NOX->Status Tests->Test 0->Test 0,Tolerance,double,1.0e-4")
        self.xmlDefaultLines.append("Charon->Solution Control->NOX->Status Tests->Test 0->Test 1,Test Type,string,NormWRMS")
        self.xmlDefaultLines.append("Charon->Solution Control->NOX->Status Tests->Test 0->Test 1,Tolerance,double,1.0")
        self.xmlDefaultLines.append("Charon->Solution Control->NOX->Status Tests->Test 0->Test 1,Relative Tolerance,double,1.0e-07")
        self.xmlDefaultLines.append("Charon->Solution Control->NOX->Status Tests->Test 0->Test 1,Absolute Tolerance,double,1.0e-14")
        self.xmlDefaultLines.append("Charon->Solution Control->NOX->Status Tests->Test 0->Test 1,Alpha,double,0.0")
        self.xmlDefaultLines.append("Charon->Solution Control->NOX->Status Tests->Test 0->Test 1,Beta,double,1.0")
        self.xmlDefaultLines.append("Charon->Solution Control->Tempus->Default Stepper->Default Solver->NOX->Status Tests->Test 0,Test Type,string,Combo")
        self.xmlDefaultLines.append("Charon->Solution Control->Tempus->Default Stepper->Default Solver->NOX->Status Tests->Test 0,Combo Type,string,AND")
        self.xmlDefaultLines.append("Charon->Solution Control->Tempus->Default Stepper->Default Solver->NOX->Status Tests->Test 0,Number of Tests,int,2")
        self.xmlDefaultLines.append("Charon->Solution Control->Tempus->Default Stepper->Default Solver->NOX->Status Tests->Test 0->Test 0,Test Type,string,NormF")
        self.xmlDefaultLines.append("Charon->Solution Control->Tempus->Default Stepper->Default Solver->NOX->Status Tests->Test 0->Test 0,Tolerance,double,1.0e-4")
        self.xmlDefaultLines.append("Charon->Solution Control->Tempus->Default Stepper->Default Solver->NOX->Status Tests->Test 0->Test 1,Test Type,string,NormWRMS")
        self.xmlDefaultLines.append("Charon->Solution Control->Tempus->Default Stepper->Default Solver->NOX->Status Tests->Test 0->Test 1,Tolerance,double,1.0")
        self.xmlDefaultLines.append("Charon->Solution Control->Tempus->Default Stepper->Default Solver->NOX->Status Tests->Test 0->Test 1,Relative Tolerance,double,1.0e-07")
        self.xmlDefaultLines.append("Charon->Solution Control->Tempus->Default Stepper->Default Solver->NOX->Status Tests->Test 0->Test 1,Absolute Tolerance,double,1.0e-14")
        self.xmlDefaultLines.append("Charon->Solution Control->Tempus->Default Stepper->Default Solver->NOX->Status Tests->Test 0->Test 1,Alpha,double,0.0")
        self.xmlDefaultLines.append("Charon->Solution Control->Tempus->Default Stepper->Default Solver->NOX->Status Tests->Test 0->Test 1,Beta,double,1.0")
        self.xmlDefaultLines.append("Charon->Solution Control->NOX->Status Tests->Test 1,Maximum Iterations,int,30")
        self.xmlDefaultLines.append("Charon->Solution Control->Tempus->Default Stepper->Default Solver->NOX->Status Tests->Test 1,Maximum Iterations,int,30")
        self.xmlDefaultLinePriority.append(1)
        self.xmlDefaultLinePriority.append(1)
        self.xmlDefaultLinePriority.append(1)
        self.xmlDefaultLinePriority.append(1)
        self.xmlDefaultLinePriority.append(1)
        self.xmlDefaultLinePriority.append(1)
        self.xmlDefaultLinePriority.append(1)
        self.xmlDefaultLinePriority.append(1)
        self.xmlDefaultLinePriority.append(1)
        self.xmlDefaultLinePriority.append(1)
        self.xmlDefaultLinePriority.append(1)
        self.xmlDefaultLinePriority.append(1)
        self.xmlDefaultLinePriority.append(1)
        self.xmlDefaultLinePriority.append(1)
        self.xmlDefaultLinePriority.append(1)
        self.xmlDefaultLinePriority.append(1)
        self.xmlDefaultLinePriority.append(1)
        self.xmlDefaultLinePriority.append(1)
        self.xmlDefaultLinePriority.append(1)
        self.xmlDefaultLinePriority.append(1)
        self.xmlDefaultLinePriority.append(1)
        self.xmlDefaultLinePriority.append(1)
        self.xmlDefaultLinePriority.append(1)
        self.xmlDefaultLinePriority.append(1)
        self.xmlDefaultLinePriority.append(1)
        self.xmlDefaultLinePriority.append(1)
        self.xmlDefaultLinePriority.append(1)
        self.xmlDefaultLinePriority.append(1)
        self.xmlDefaultLinePriority.append(1)

        self.xmlReturned = []
        self.xmlPriorityCode = []
        self.specialInformation = []



    def isThisMe(self,tokenizer,line):
        affirmPrimary = False
        for index,fO in enumerate(self.foundOptions):
            self.foundOptions[index] = False
        self.primaryVariables = {}
        self.optionVariables = []*len(self.options)
        (affirmPrimary,self.foundOptions,self.primaryVariables,self.optionVariables,suggestedSyntax) = tokenizer.isItMe(line,self.primary,self.options)
        return (affirmPrimary,suggestedSyntax)



    def getName(self):
        # Return parser name
         return self.parserName



    def getHelp(self,verbosity):
        # Return help content
        if verbosity.lower() == "long":
            return (self.interpreterHelpLine,self.interpreterLongHelp)
        else:
            return (self.interpreterHelpLine,self.interpreterQuickHelp)



    def generateXML(self,tokenizer,line):
        # Tokenize the line
        lineTokens = tokenizer.tokenize(line)
        self.xmlNewRequiredLines[:] = []
        for xL in self.xmlRequiredLines:
            self.xmlNewRequiredLines.append(xL)

        for key in self.primaryVariables:
            for nRLIndex,nRL in enumerate(self.xmlNewRequiredLines):
                self.xmlNewRequiredLines[nRLIndex] = nRL.replace(key,self.primaryVariables[key])
            for siIndex, siValue in enumerate(self.specialInformation):
                self.specialInformation[siIndex] = [item.replace(key,self.primaryVariables[key]) for item in siValue]

        for index,xmlLine in enumerate(self.xmlNewRequiredLines):
            self.xmlReturned.append(xmlLine)
            self.xmlPriorityCode.append(self.xmlRequiredLinePriority[index]) #required lines have priority code 2
        for optIndex,fO in enumerate(self.foundOptions):
            if fO == True:
                self.Returned=copy.deepcopy(self.xmlOptionalLines[optIndex])
                for key in self.primaryVariables:
                    for retIndex,xOL in enumerate(self.Returned):
                        self.Returned[retIndex] = xOL.replace(key,self.primaryVariables[key])
                for key in self.optionVariables[optIndex]:
                    for retIndex,xOL in enumerate(self.Returned):
                        self.Returned[retIndex] = xOL.replace(key,self.optionVariables[optIndex][key])
                        for siIndex, siValue in enumerate(self.specialInformation):
                            self.specialInformation[siIndex] = [item.replace(key,self.optionVariables[optIndex][key]) for item in siValue]
                for retIndex,ret in enumerate(self.Returned):
                    self.xmlReturned.append(ret)
                    self.xmlPriorityCode.append(self.xmlOptionalLinePriority[optIndex][retIndex])

        for xmlLine in self.xmlDefaultLines:
            self.xmlReturned.append(xmlLine)
            self.xmlPriorityCode.append(1) #optional lines have priority code 1

        return (self.xmlReturned,self.xmlPriorityCode,self.specialInformation)



