
import copy


class charonLineParserContactOnInsulator:
    "ContactOnInsulator parser"

    def __init__(self):
        # Register the parsing keys
        self.parserName = "ContactOnInsulator"
        self.primary = "bc is contact on insulator for {sidesetID} on {geometryBlock} with work function {workFunction}"
        self.options = []
        self.options.append("fixed at {potential}")
        self.options.append("swept from {potential1} to {potential2}")
        self.options.append("linearly ramped from {time0} to {time1} and voltage from {voltage0} to {voltage1}")
        self.options.append("trapezoid pulse with amplitude = {amplitude} period = {period} rise time = {risetime} fall time = {falltime}")
        self.options.append("duty cycle = {dutycycle}")
        self.options.append("dc offset = {DCoffset}")
        self.options.append("delay = {delay}")
        self.options.append("repeat = {repeat}")
        self.foundOptions = []
        for index,opt in enumerate(self.options):
            self.foundOptions.append(False)
        self.primaryVariables = []
        self.optionVariables = [[]]*len(self.options)
        self.interpreterHelpLine = "BC is contact on insulator for {sidesetID} on {geometryBlock} with work function {workFunction} [fixed at {potential}[ swept from {potential1} to {potential2} [ linearly ramped from {time0} to {time1} and voltage from {voltage0} to {voltage1}[ trapezoid pulse with amplitude = {amplitude} period = {period} rise time = {risetime} fall time = {falltime} [duty cycle = {dutycycle} [dc offset = {DCoffset} [delay = {delay} [repeat = {repeat}]]]]]]]] "
        self.interpreterQuickHelp = "Specify the potential on a contact"
        self.interpreterLongHelp = "Specify the potential on a contact. <> sidesetID is the contact name/type <> geometryBlock is the geometry name the contact is attached to <> potential is the value in volts"

        # Register the xml required lines
        self.xmlRequiredLines = []
        self.xmlRequiredLinePriority = []
        self.xmlRequiredLines.append("Charon->Boundary Conditions->{sidesetID}ANONYMOUS,Type,string,Dirichlet")
        self.xmlRequiredLines.append("Charon->Boundary Conditions->{sidesetID}ANONYMOUS,Sideset ID,string,{sidesetID}")
        self.xmlRequiredLines.append("Charon->Boundary Conditions->{sidesetID}ANONYMOUS,Element Block ID,string,{geometryBlock}")
        self.xmlRequiredLines.append("Charon->Boundary Conditions->{sidesetID}ANONYMOUS,Equation Set Name,string,ELECTRIC_POTENTIAL")
        self.xmlRequiredLines.append("Charon->Boundary Conditions->{sidesetID}ANONYMOUS,Strategy,string,Contact On Insulator")
        self.xmlRequiredLines.append("Charon->Boundary Conditions->{sidesetID}ANONYMOUS->Data,Work Function,double,{workFunction}")
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlNewRequiredLines = []

        # Register the xml optional lines
        self.xmlOptionalLines = [[]]
        self.xmlOptionalLinePriority = [[]]
        self.xmlOptionalLines[0].append(" Charon->Boundary Conditions->{sidesetID}ANONYMOUS->Data,Voltage,double,{potential}")
        self.xmlOptionalLines.append([])
        self.xmlOptionalLines[1].append(" Charon->Boundary Conditions->{sidesetID}ANONYMOUS->Data,Varying Voltage,string,Parameter")
        self.xmlOptionalLines[1].append(" Charon->Solution Control,Piro Solver,string,LOCA")
        self.xmlOptionalLines[1].append(" Charon->Solution Control->LOCA->Predictor,Method,string,Constant")
        self.xmlOptionalLines[1].append(" Charon->Solution Control->LOCA->Stepper,Initial Value,double,{potential1}")
        self.xmlOptionalLines[1].append(" Charon->Solution Control->LOCA->Stepper,Continuation Parameter,string,Varying Voltage")
        self.xmlOptionalLines[1].append(" Charon->Solution Control->LOCA->Stepper,Max Steps,int,1000")
        self.xmlOptionalLines[1].append(" Charon->Solution Control->LOCA->Stepper,Max Value,double,{potential2}")
        self.xmlOptionalLines[1].append(" Charon->Solution Control->LOCA->Stepper,Min Value,double,{potential1}")
        self.xmlOptionalLines[1].append(" Charon->Solution Control->LOCA->Stepper,Compute Eigenvalues,bool,0")
        self.xmlOptionalLines[1].append(" Charon->Solution Control->LOCA->Step Size,Initial Step Size,double,1.0")
        self.xmlOptionalLines[1].append(" Charon->Active Parameters,Number of Parameter Vectors,int,1")
        self.xmlOptionalLines[1].append(" Charon->Active Parameters->Parameter Vector 0,Number,int,1")
        self.xmlOptionalLines[1].append(" Charon->Active Parameters->Parameter Vector 0,Parameter 0,string,Varying Voltage")
        self.xmlOptionalLines[1].append(" Charon->Active Parameters->Parameter Vector 0,Initial Value 0,double,{potential1}")
        self.xmlOptionalLines[1].append(" use Modifier 0")
        self.xmlOptionalLines.append([])
        self.xmlOptionalLines[2].append(" Charon->Boundary Conditions->{sidesetID}ANONYMOUS->Data->Linear Ramp,Initial Time,double,{time0}")
        self.xmlOptionalLines[2].append(" Charon->Boundary Conditions->{sidesetID}ANONYMOUS->Data->Linear Ramp,Final Time,double,{time1}")
        self.xmlOptionalLines[2].append(" Charon->Boundary Conditions->{sidesetID}ANONYMOUS->Data->Linear Ramp,Initial Voltage,double,{voltage0}")
        self.xmlOptionalLines[2].append(" Charon->Boundary Conditions->{sidesetID}ANONYMOUS->Data->Linear Ramp,Final Voltage,double,{voltage1}")
        self.xmlOptionalLines.append([])
        self.xmlOptionalLines[3].append(" Charon->Boundary Conditions->{sidesetID}ANONYMOUS->Data->Trapezoid Pulse,Amplitude,double,{amplitude}")
        self.xmlOptionalLines[3].append(" Charon->Boundary Conditions->{sidesetID}ANONYMOUS->Data->Trapezoid Pulse,Period,double,{period}")
        self.xmlOptionalLines[3].append(" Charon->Boundary Conditions->{sidesetID}ANONYMOUS->Data->Trapezoid Pulse,Rise Time,double,{risetime}")
        self.xmlOptionalLines[3].append(" Charon->Boundary Conditions->{sidesetID}ANONYMOUS->Data->Trapezoid Pulse,Fall Time,double,{falltime}")
        self.xmlOptionalLines.append([])
        self.xmlOptionalLines[4].append(" Charon->Boundary Conditions->{sidesetID}ANONYMOUS->Data->Trapezoid Pulse,Duty Cycle,double,{dutycycle}")
        self.xmlOptionalLines.append([])
        self.xmlOptionalLines[5].append(" Charon->Boundary Conditions->{sidesetID}ANONYMOUS->Data->Trapezoid Pulse,DC Offset,double,{DCoffset}")
        self.xmlOptionalLines.append([])
        self.xmlOptionalLines[6].append(" Charon->Boundary Conditions->{sidesetID}ANONYMOUS->Data->Trapezoid Pulse,Delay,double,{delay}")
        self.xmlOptionalLines.append([])
        self.xmlOptionalLines[7].append(" Charon->Boundary Conditions->{sidesetID}ANONYMOUS->Data->Trapezoid Pulse,Number Pulses,int,{repeat}")
        self.xmlOptionalLinePriority[0].append(2)
        self.xmlOptionalLinePriority.append([])
        self.xmlOptionalLinePriority[1].append(2)
        self.xmlOptionalLinePriority[1].append(2)
        self.xmlOptionalLinePriority[1].append(2)
        self.xmlOptionalLinePriority[1].append(2)
        self.xmlOptionalLinePriority[1].append(2)
        self.xmlOptionalLinePriority[1].append(2)
        self.xmlOptionalLinePriority[1].append(2)
        self.xmlOptionalLinePriority[1].append(2)
        self.xmlOptionalLinePriority[1].append(2)
        self.xmlOptionalLinePriority[1].append(2)
        self.xmlOptionalLinePriority[1].append(2)
        self.xmlOptionalLinePriority[1].append(2)
        self.xmlOptionalLinePriority[1].append(2)
        self.xmlOptionalLinePriority[1].append(2)
        self.xmlOptionalLinePriority[1].append(2)
        self.xmlOptionalLinePriority.append([])
        self.xmlOptionalLinePriority[2].append(2)
        self.xmlOptionalLinePriority[2].append(2)
        self.xmlOptionalLinePriority[2].append(2)
        self.xmlOptionalLinePriority[2].append(2)
        self.xmlOptionalLinePriority.append([])
        self.xmlOptionalLinePriority[3].append(2)
        self.xmlOptionalLinePriority[3].append(2)
        self.xmlOptionalLinePriority[3].append(2)
        self.xmlOptionalLinePriority[3].append(2)
        self.xmlOptionalLinePriority.append([])
        self.xmlOptionalLinePriority[4].append(2)
        self.xmlOptionalLinePriority.append([])
        self.xmlOptionalLinePriority[5].append(2)
        self.xmlOptionalLinePriority.append([])
        self.xmlOptionalLinePriority[6].append(2)
        self.xmlOptionalLinePriority.append([])
        self.xmlOptionalLinePriority[7].append(2)

        # Register the xml default lines
        self.xmlDefaultLines = []
        self.xmlDefaultLinePriority = []

        self.xmlReturned = []
        self.xmlPriorityCode = []
        self.specialInformation = []



    def isThisMe(self,tokenizer,line):
        affirmPrimary = False
        for index,fO in enumerate(self.foundOptions):
            self.foundOptions[index] = False
        self.primaryVariables = {}
        self.optionVariables = []*len(self.options)
        (affirmPrimary,self.foundOptions,self.primaryVariables,self.optionVariables,suggestedSyntax) = tokenizer.isItMe(line,self.primary,self.options)
        return (affirmPrimary,suggestedSyntax)



    def getName(self):
        # Return parser name
         return self.parserName



    def getHelp(self,verbosity):
        # Return help content
        if verbosity.lower() == "long":
            return (self.interpreterHelpLine,self.interpreterLongHelp)
        else:
            return (self.interpreterHelpLine,self.interpreterQuickHelp)



    def generateXML(self,tokenizer,line):
        # Tokenize the line
        lineTokens = tokenizer.tokenize(line)
        self.xmlNewRequiredLines[:] = []
        for xL in self.xmlRequiredLines:
            self.xmlNewRequiredLines.append(xL)

        for key in self.primaryVariables:
            for nRLIndex,nRL in enumerate(self.xmlNewRequiredLines):
                self.xmlNewRequiredLines[nRLIndex] = nRL.replace(key,self.primaryVariables[key])
            for siIndex, siValue in enumerate(self.specialInformation):
                self.specialInformation[siIndex] = [item.replace(key,self.primaryVariables[key]) for item in siValue]

        for index,xmlLine in enumerate(self.xmlNewRequiredLines):
            self.xmlReturned.append(xmlLine)
            self.xmlPriorityCode.append(self.xmlRequiredLinePriority[index]) #required lines have priority code 2
        for optIndex,fO in enumerate(self.foundOptions):
            if fO == True:
                self.Returned=copy.deepcopy(self.xmlOptionalLines[optIndex])
                for key in self.primaryVariables:
                    for retIndex,xOL in enumerate(self.Returned):
                        self.Returned[retIndex] = xOL.replace(key,self.primaryVariables[key])
                for key in self.optionVariables[optIndex]:
                    for retIndex,xOL in enumerate(self.Returned):
                        self.Returned[retIndex] = xOL.replace(key,self.optionVariables[optIndex][key])
                        for siIndex, siValue in enumerate(self.specialInformation):
                            self.specialInformation[siIndex] = [item.replace(key,self.optionVariables[optIndex][key]) for item in siValue]
                for retIndex,ret in enumerate(self.Returned):
                    self.xmlReturned.append(ret)
                    self.xmlPriorityCode.append(self.xmlOptionalLinePriority[optIndex][retIndex])

        for xmlLine in self.xmlDefaultLines:
            self.xmlReturned.append(xmlLine)
            self.xmlPriorityCode.append(1) #optional lines have priority code 1

        return (self.xmlReturned,self.xmlPriorityCode,self.specialInformation)



