
import copy


class charonLineParserInitialConditions:
    "InitialConditions parser"

    def __init__(self):
        # Register the parsing keys
        self.parserName = "InitialConditions"
        self.primary = "initial conditions for {DOF} in {geometryBlock} is"
        self.options = []
        self.options.append("exodus file")
        self.options.append("equilibrium density")
        self.options.append("equilibrium potential")
        self.options.append("equilibrium potential with semblocks = {semids}")
        self.options.append("constant = {constValue}")
        self.options.append("effective bandgap and electron affinity = {bgval} and {affin}")
        self.options.append("uninitialized")
        self.foundOptions = []
        for index,opt in enumerate(self.options):
            self.foundOptions.append(False)
        self.primaryVariables = []
        self.optionVariables = [[]]*len(self.options)
        self.interpreterHelpLine = "Initial Conditions for {DOF} in {geometryBlock} is [ Exodus File[ equilibrium density [ equilibrium potential [ equilibrium potential with semblocks = {semids} [constant = {constValue} [ effective bandgap and electron affinity = {bgval} and {affin} [uninitialized ]]]]]]] "
        self.interpreterQuickHelp = "Specify the source of the initial condition for the degrees of freedom"
        self.interpreterLongHelp = "Specify the source of the initial condition for the degrees of freedom <> Degrees of freedoms are <> ELECTRIC_POTENTIAL <> ELECTRON_DENSITY <> HOLE_DENSITY <> initial conditions are: <> Exodus File <> Equilibrium Density"

        # Register the xml required lines
        self.xmlRequiredLines = []
        self.xmlRequiredLinePriority = []
        self.xmlNewRequiredLines = []

        # Register the xml optional lines
        self.xmlOptionalLines = [[]]
        self.xmlOptionalLinePriority = [[]]
        self.xmlOptionalLines[0].append(" Charon->Initial Conditions->{geometryBlock}->{DOF},Value,string,Exodus File")
        self.xmlOptionalLines[0].append(" use Modifier 1")
        self.xmlOptionalLines.append([])
        self.xmlOptionalLines[1].append(" Charon->Initial Conditions->{geometryBlock}->{DOF},Value,string,Equilibrium Density")
        self.xmlOptionalLines.append([])
        self.xmlOptionalLines[2].append(" Charon->Initial Conditions->{geometryBlock}->{DOF},Value,string,Equilibrium Potential")
        self.xmlOptionalLines.append([])
        self.xmlOptionalLines[3].append(" Charon->Initial Conditions->{geometryBlock}->{DOF},Value,string,Equilibrium Potential with SemBlockIds = ({semids})")
        self.xmlOptionalLines.append([])
        self.xmlOptionalLines[4].append(" Charon->Initial Conditions->{geometryBlock}->{DOF},Value,double,{constValue}")
        self.xmlOptionalLines[4].append(" use Modifier 0")
        self.xmlOptionalLines.append([])
        self.xmlOptionalLines[5].append(" Charon->Initial Conditions->{geometryBlock}->{DOF},Value,string,Equilibrium Density")
        self.xmlOptionalLines[5].append(" Charon->Initial Conditions->{geometryBlock}->{DOF},Effective Band Gap,double,{bgval}")
        self.xmlOptionalLines[5].append(" Charon->Initial Conditions->{geometryBlock}->{DOF},Effective Electron Affinity,double,{affin}")
        self.xmlOptionalLines.append([])
        self.xmlOptionalLines[6].append(" Charon->Initial Conditions->{geometryBlock}->{DOF},Value,double,0.0")
        self.xmlOptionalLinePriority[0].append(2)
        self.xmlOptionalLinePriority[0].append(2)
        self.xmlOptionalLinePriority.append([])
        self.xmlOptionalLinePriority[1].append(2)
        self.xmlOptionalLinePriority.append([])
        self.xmlOptionalLinePriority[2].append(2)
        self.xmlOptionalLinePriority.append([])
        self.xmlOptionalLinePriority[3].append(2)
        self.xmlOptionalLinePriority.append([])
        self.xmlOptionalLinePriority[4].append(2)
        self.xmlOptionalLinePriority[4].append(2)
        self.xmlOptionalLinePriority.append([])
        self.xmlOptionalLinePriority[5].append(2)
        self.xmlOptionalLinePriority[5].append(2)
        self.xmlOptionalLinePriority[5].append(2)
        self.xmlOptionalLinePriority.append([])
        self.xmlOptionalLinePriority[6].append(2)

        # Register the xml default lines
        self.xmlDefaultLines = []
        self.xmlDefaultLinePriority = []

        self.xmlReturned = []
        self.xmlPriorityCode = []
        self.specialInformation = []



    def isThisMe(self,tokenizer,line):
        affirmPrimary = False
        for index,fO in enumerate(self.foundOptions):
            self.foundOptions[index] = False
        self.primaryVariables = {}
        self.optionVariables = []*len(self.options)
        (affirmPrimary,self.foundOptions,self.primaryVariables,self.optionVariables,suggestedSyntax) = tokenizer.isItMe(line,self.primary,self.options)
        return (affirmPrimary,suggestedSyntax)



    def getName(self):
        # Return parser name
         return self.parserName



    def getHelp(self,verbosity):
        # Return help content
        if verbosity.lower() == "long":
            return (self.interpreterHelpLine,self.interpreterLongHelp)
        else:
            return (self.interpreterHelpLine,self.interpreterQuickHelp)



    def generateXML(self,tokenizer,line):
        # Tokenize the line
        lineTokens = tokenizer.tokenize(line)
        self.xmlNewRequiredLines[:] = []
        for xL in self.xmlRequiredLines:
            self.xmlNewRequiredLines.append(xL)

        for key in self.primaryVariables:
            for nRLIndex,nRL in enumerate(self.xmlNewRequiredLines):
                self.xmlNewRequiredLines[nRLIndex] = nRL.replace(key,self.primaryVariables[key])
            for siIndex, siValue in enumerate(self.specialInformation):
                self.specialInformation[siIndex] = [item.replace(key,self.primaryVariables[key]) for item in siValue]

        for index,xmlLine in enumerate(self.xmlNewRequiredLines):
            self.xmlReturned.append(xmlLine)
            self.xmlPriorityCode.append(self.xmlRequiredLinePriority[index]) #required lines have priority code 2
        for optIndex,fO in enumerate(self.foundOptions):
            if fO == True:
                self.Returned=copy.deepcopy(self.xmlOptionalLines[optIndex])
                for key in self.primaryVariables:
                    for retIndex,xOL in enumerate(self.Returned):
                        self.Returned[retIndex] = xOL.replace(key,self.primaryVariables[key])
                for key in self.optionVariables[optIndex]:
                    for retIndex,xOL in enumerate(self.Returned):
                        self.Returned[retIndex] = xOL.replace(key,self.optionVariables[optIndex][key])
                        for siIndex, siValue in enumerate(self.specialInformation):
                            self.specialInformation[siIndex] = [item.replace(key,self.optionVariables[optIndex][key]) for item in siValue]
                for retIndex,ret in enumerate(self.Returned):
                    self.xmlReturned.append(ret)
                    self.xmlPriorityCode.append(self.xmlOptionalLinePriority[optIndex][retIndex])

        for xmlLine in self.xmlDefaultLines:
            self.xmlReturned.append(xmlLine)
            self.xmlPriorityCode.append(1) #optional lines have priority code 1

        return (self.xmlReturned,self.xmlPriorityCode,self.specialInformation)



