
import copy


class charonLineParserMixedModeCurrentBC:
    "MixedModeCurrentBC parser"

    def __init__(self):
        # Register the parsing keys
        self.parserName = "MixedModeCurrentBC"
        self.primary = "bc is mixed mode via current for {sidesetID} on {geometryBlock} as node named {nodeName} in netlist {netlistName} with initial current {initCurrent} with initial voltage {initVoltage}"
        self.options = []
        self.options.append("with area {cntarea}")
        self.options.append("and length {cntlength}")
        self.options.append("with base doping type {baseType}")
        self.foundOptions = []
        for index,opt in enumerate(self.options):
            self.foundOptions.append(False)
        self.primaryVariables = []
        self.optionVariables = [[]]*len(self.options)
        self.interpreterHelpLine = "BC is mixed mode via current for {sidesetID} on {geometryBlock} as node named {nodeName} in netlist {netlistName} with initial current {initCurrent} with initial voltage {initVoltage} [with area {cntarea} [ and length {cntlength} [ with base doping type {baseType} ]]] "
        self.interpreterQuickHelp = "Specify a mixed mode Xyce-coupled current boundary condition"
        self.interpreterLongHelp = "Specify a mixed mode Xyce-coupled current boundary condition  <> sidesetID is the contact name/type <> geometryBlock is the geometry name the contact is attached to <> nodeName is the name of the node in the Xyce netlist <> netlistName is the Xyce netlist <> initCurrent and initVoltage are the initialized  current and voltage couple values"

        # Register the xml required lines
        self.xmlRequiredLines = []
        self.xmlRequiredLinePriority = []
        self.xmlRequiredLines.append("Charon->Boundary Conditions->{sidesetID}ANONYMOUS,Type,string,Dirichlet")
        self.xmlRequiredLines.append("Charon->Boundary Conditions->{sidesetID}ANONYMOUS,Sideset ID,string,{sidesetID}")
        self.xmlRequiredLines.append("Charon->Boundary Conditions->{sidesetID}ANONYMOUS,Element Block ID,string,{geometryBlock}")
        self.xmlRequiredLines.append("Charon->Boundary Conditions->{sidesetID}ANONYMOUS,Equation Set Name,string,ALL_DOFS")
        self.xmlRequiredLines.append("Charon->Boundary Conditions->{sidesetID}ANONYMOUS,Strategy,string,Mixed Mode via Current")
        self.xmlRequiredLines.append("Charon->Boundary Conditions->{sidesetID}ANONYMOUS->Data,Netlist,string,{netlistName}")
        self.xmlRequiredLines.append("Charon->Boundary Conditions->{sidesetID}ANONYMOUS->Data,Node Name,string,{nodeName}")
        self.xmlRequiredLines.append("Charon->Boundary Conditions->{sidesetID}ANONYMOUS->Data,Current Value,double,{initCurrent}")
        self.xmlRequiredLines.append("Charon->Boundary Conditions->{sidesetID}ANONYMOUS->Data,Initial Voltage,double,{initVoltage}")
        self.xmlRequiredLines.append("Charon->Boundary Conditions->{sidesetID}ANONYMOUS->Data,Initial Xyce Coupling on Step Number,int,3")
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlNewRequiredLines = []

        # Register the xml optional lines
        self.xmlOptionalLines = [[]]
        self.xmlOptionalLinePriority = [[]]
        self.xmlOptionalLines[0].append(" Charon->Boundary Conditions->{sidesetID}ANONYMOUS->Data,Device Contact Area,double,{cntarea}")
        self.xmlOptionalLines.append([])
        self.xmlOptionalLines[1].append(" Charon->Boundary Conditions->{sidesetID}ANONYMOUS->Data,Simulation Contact Length,double,{cntlength}")
        self.xmlOptionalLines.append([])
        self.xmlOptionalLines[2].append(" Charon->Boundary Conditions->{sidesetID}ANONYMOUS->Data,BJT1D Base Doping Type,string,{baseType}")
        self.xmlOptionalLinePriority[0].append(2)
        self.xmlOptionalLinePriority.append([])
        self.xmlOptionalLinePriority[1].append(2)
        self.xmlOptionalLinePriority.append([])
        self.xmlOptionalLinePriority[2].append(2)

        # Register the xml default lines
        self.xmlDefaultLines = []
        self.xmlDefaultLinePriority = []

        self.xmlReturned = []
        self.xmlPriorityCode = []
        self.specialInformation = []



    def isThisMe(self,tokenizer,line):
        affirmPrimary = False
        for index,fO in enumerate(self.foundOptions):
            self.foundOptions[index] = False
        self.primaryVariables = {}
        self.optionVariables = []*len(self.options)
        (affirmPrimary,self.foundOptions,self.primaryVariables,self.optionVariables,suggestedSyntax) = tokenizer.isItMe(line,self.primary,self.options)
        return (affirmPrimary,suggestedSyntax)



    def getName(self):
        # Return parser name
         return self.parserName



    def getHelp(self,verbosity):
        # Return help content
        if verbosity.lower() == "long":
            return (self.interpreterHelpLine,self.interpreterLongHelp)
        else:
            return (self.interpreterHelpLine,self.interpreterQuickHelp)



    def generateXML(self,tokenizer,line):
        # Tokenize the line
        lineTokens = tokenizer.tokenize(line)
        self.xmlNewRequiredLines[:] = []
        for xL in self.xmlRequiredLines:
            self.xmlNewRequiredLines.append(xL)

        for key in self.primaryVariables:
            for nRLIndex,nRL in enumerate(self.xmlNewRequiredLines):
                self.xmlNewRequiredLines[nRLIndex] = nRL.replace(key,self.primaryVariables[key])
            for siIndex, siValue in enumerate(self.specialInformation):
                self.specialInformation[siIndex] = [item.replace(key,self.primaryVariables[key]) for item in siValue]

        for index,xmlLine in enumerate(self.xmlNewRequiredLines):
            self.xmlReturned.append(xmlLine)
            self.xmlPriorityCode.append(self.xmlRequiredLinePriority[index]) #required lines have priority code 2
        for optIndex,fO in enumerate(self.foundOptions):
            if fO == True:
                self.Returned=copy.deepcopy(self.xmlOptionalLines[optIndex])
                for key in self.primaryVariables:
                    for retIndex,xOL in enumerate(self.Returned):
                        self.Returned[retIndex] = xOL.replace(key,self.primaryVariables[key])
                for key in self.optionVariables[optIndex]:
                    for retIndex,xOL in enumerate(self.Returned):
                        self.Returned[retIndex] = xOL.replace(key,self.optionVariables[optIndex][key])
                        for siIndex, siValue in enumerate(self.specialInformation):
                            self.specialInformation[siIndex] = [item.replace(key,self.optionVariables[optIndex][key]) for item in siValue]
                for retIndex,ret in enumerate(self.Returned):
                    self.xmlReturned.append(ret)
                    self.xmlPriorityCode.append(self.xmlOptionalLinePriority[optIndex][retIndex])

        for xmlLine in self.xmlDefaultLines:
            self.xmlReturned.append(xmlLine)
            self.xmlPriorityCode.append(1) #optional lines have priority code 1

        return (self.xmlReturned,self.xmlPriorityCode,self.specialInformation)



