
import copy


class charonLineParserTimeSinusoidalBC:
    "TimeSinusoidalBC parser"

    def __init__(self):
        # Register the parsing keys
        self.parserName = "TimeSinusoidalBC"
        self.primary = "bc is sinusoidal for {sidesetID} on {geometryBlock}"
        self.options = []
        self.options.append("with ampl = {amplitude1}")
        self.options.append("freq = {frequency1}")
        self.options.append("phase = {phase1}")
        self.options.append("plus ampl2 = {amplitude2}")
        self.options.append("freq2 = {frequency2}")
        self.options.append("phase2 = {phase2}")
        self.options.append("and dc offset = {DCoffset}")
        self.options.append("with fermi pinning and contact ion density = {ionDensity}")
        self.foundOptions = []
        for index,opt in enumerate(self.options):
            self.foundOptions.append(False)
        self.primaryVariables = []
        self.optionVariables = [[]]*len(self.options)
        self.interpreterHelpLine = "BC is sinusoidal for {sidesetID} on {geometryBlock} [with ampl = {amplitude1} [freq = {frequency1} [phase = {phase1} [plus ampl2 = {amplitude2} [freq2 = {frequency2} [phase2 = {phase2} [and dc offset = {DCoffset} [ with fermi pinning and contact ion density = {ionDensity} ] ]]]]]]] "
        self.interpreterQuickHelp = "Specify a temporally sinusoidal voltage applied on a contact."
        self.interpreterLongHelp = "Specify a temporally sinusoidal voltage applied on a contact  which can be specified as a superposition of two sinusoids and a DC voltage. <> sidesetID is the contact name/type <> geometryBlock is the geometry block the contact is attached to <> amplitude1 and amplitude2 are the amplitudes of the applied sinusoid voltages in Volts. <> frequency1 and frequency2 are the frequencies of the applied sinusoid voltages  in Hz. <> phase1 and phase2 are the phase shifts of the applied sinusoid voltages in a percentage of 2\pi and so their values are within [0  1]. <> DCoffset is the DC voltage applied."

        # Register the xml required lines
        self.xmlRequiredLines = []
        self.xmlRequiredLinePriority = []
        self.xmlRequiredLines.append("Charon->Boundary Conditions->{sidesetID}ANONYMOUS,Type,string,Dirichlet")
        self.xmlRequiredLines.append("Charon->Boundary Conditions->{sidesetID}ANONYMOUS,Sideset ID,string,{sidesetID}")
        self.xmlRequiredLines.append("Charon->Boundary Conditions->{sidesetID}ANONYMOUS,Element Block ID,string,{geometryBlock}")
        self.xmlRequiredLines.append("Charon->Boundary Conditions->{sidesetID}ANONYMOUS,Equation Set Name,string,ALL_DOFS")
        self.xmlRequiredLines.append("Charon->Boundary Conditions->{sidesetID}ANONYMOUS,Strategy,string,Sinusoid")
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlRequiredLinePriority.append(2)
        self.xmlNewRequiredLines = []

        # Register the xml optional lines
        self.xmlOptionalLines = [[]]
        self.xmlOptionalLinePriority = [[]]
        self.xmlOptionalLines[0].append(" Charon->Boundary Conditions->{sidesetID}ANONYMOUS->Data,Amplitude 1,double,{amplitude1}")
        self.xmlOptionalLines.append([])
        self.xmlOptionalLines[1].append(" Charon->Boundary Conditions->{sidesetID}ANONYMOUS->Data,Frequency 1,double,{frequency1}")
        self.xmlOptionalLines.append([])
        self.xmlOptionalLines[2].append(" Charon->Boundary Conditions->{sidesetID}ANONYMOUS->Data,Phase Shift 1,double,{phase1}")
        self.xmlOptionalLines.append([])
        self.xmlOptionalLines[3].append(" Charon->Boundary Conditions->{sidesetID}ANONYMOUS->Data,Amplitude 2,double,{amplitude2}")
        self.xmlOptionalLines.append([])
        self.xmlOptionalLines[4].append(" Charon->Boundary Conditions->{sidesetID}ANONYMOUS->Data,Frequency 2,double,{frequency2}")
        self.xmlOptionalLines.append([])
        self.xmlOptionalLines[5].append(" Charon->Boundary Conditions->{sidesetID}ANONYMOUS->Data,Phase Shift 2,double,{phase2}")
        self.xmlOptionalLines.append([])
        self.xmlOptionalLines[6].append(" Charon->Boundary Conditions->{sidesetID}ANONYMOUS->Data,DC Offset,double,{DCoffset}")
        self.xmlOptionalLines.append([])
        self.xmlOptionalLines[7].append(" Charon->Boundary Conditions->{sidesetID}ANONYMOUS->Data,Fermi Level Pinning,bool,true")
        self.xmlOptionalLines[7].append(" Charon->Boundary Conditions->{sidesetID}ANONYMOUS->Data,Contact Ion Density,double,{ionDensity}")
        self.xmlOptionalLinePriority[0].append(2)
        self.xmlOptionalLinePriority.append([])
        self.xmlOptionalLinePriority[1].append(2)
        self.xmlOptionalLinePriority.append([])
        self.xmlOptionalLinePriority[2].append(2)
        self.xmlOptionalLinePriority.append([])
        self.xmlOptionalLinePriority[3].append(2)
        self.xmlOptionalLinePriority.append([])
        self.xmlOptionalLinePriority[4].append(2)
        self.xmlOptionalLinePriority.append([])
        self.xmlOptionalLinePriority[5].append(2)
        self.xmlOptionalLinePriority.append([])
        self.xmlOptionalLinePriority[6].append(2)
        self.xmlOptionalLinePriority.append([])
        self.xmlOptionalLinePriority[7].append(2)
        self.xmlOptionalLinePriority[7].append(2)

        # Register the xml default lines
        self.xmlDefaultLines = []
        self.xmlDefaultLinePriority = []

        self.xmlReturned = []
        self.xmlPriorityCode = []
        self.specialInformation = []



    def isThisMe(self,tokenizer,line):
        affirmPrimary = False
        for index,fO in enumerate(self.foundOptions):
            self.foundOptions[index] = False
        self.primaryVariables = {}
        self.optionVariables = []*len(self.options)
        (affirmPrimary,self.foundOptions,self.primaryVariables,self.optionVariables,suggestedSyntax) = tokenizer.isItMe(line,self.primary,self.options)
        return (affirmPrimary,suggestedSyntax)



    def getName(self):
        # Return parser name
         return self.parserName



    def getHelp(self,verbosity):
        # Return help content
        if verbosity.lower() == "long":
            return (self.interpreterHelpLine,self.interpreterLongHelp)
        else:
            return (self.interpreterHelpLine,self.interpreterQuickHelp)



    def generateXML(self,tokenizer,line):
        # Tokenize the line
        lineTokens = tokenizer.tokenize(line)
        self.xmlNewRequiredLines[:] = []
        for xL in self.xmlRequiredLines:
            self.xmlNewRequiredLines.append(xL)

        for key in self.primaryVariables:
            for nRLIndex,nRL in enumerate(self.xmlNewRequiredLines):
                self.xmlNewRequiredLines[nRLIndex] = nRL.replace(key,self.primaryVariables[key])
            for siIndex, siValue in enumerate(self.specialInformation):
                self.specialInformation[siIndex] = [item.replace(key,self.primaryVariables[key]) for item in siValue]

        for index,xmlLine in enumerate(self.xmlNewRequiredLines):
            self.xmlReturned.append(xmlLine)
            self.xmlPriorityCode.append(self.xmlRequiredLinePriority[index]) #required lines have priority code 2
        for optIndex,fO in enumerate(self.foundOptions):
            if fO == True:
                self.Returned=copy.deepcopy(self.xmlOptionalLines[optIndex])
                for key in self.primaryVariables:
                    for retIndex,xOL in enumerate(self.Returned):
                        self.Returned[retIndex] = xOL.replace(key,self.primaryVariables[key])
                for key in self.optionVariables[optIndex]:
                    for retIndex,xOL in enumerate(self.Returned):
                        self.Returned[retIndex] = xOL.replace(key,self.optionVariables[optIndex][key])
                        for siIndex, siValue in enumerate(self.specialInformation):
                            self.specialInformation[siIndex] = [item.replace(key,self.optionVariables[optIndex][key]) for item in siValue]
                for retIndex,ret in enumerate(self.Returned):
                    self.xmlReturned.append(ret)
                    self.xmlPriorityCode.append(self.xmlOptionalLinePriority[optIndex][retIndex])

        for xmlLine in self.xmlDefaultLines:
            self.xmlReturned.append(xmlLine)
            self.xmlPriorityCode.append(1) #optional lines have priority code 1

        return (self.xmlReturned,self.xmlPriorityCode,self.specialInformation)



